/** @file ag_config_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.04.08

	@brief [ ag_config_dlg ]
*/

#include "ag_config_dlg.h"
#include <agar/core/types.h>
#include "../emu.h"
#include "../config.h"
#include "ag_gui_base.h"

AG_CONFIG_DLG::AG_CONFIG_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(&param, 0, sizeof(param));
	param.fdd_type  = config.fdd_type;

	param.wav_reverse = 0;
	param.wav_half = 1;
	param.wav_correct = 0;
	param.wav_correct_type = 0;

	param.wav_sample_rate = 3;
	param.wav_sample_bits = 0;

	param.io_port = config.io_port;

	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;

	paramtmp = param;
}

AG_CONFIG_DLG::~AG_CONFIG_DLG()
{
}

/**
 * create config dialog
 */
void AG_CONFIG_DLG::Create()
{
	paramtmp = param;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, "Configure");

	AG_Box *hbox_base;
	AG_Box *vbox_left;
	AG_Box *vbox_right;
	AG_Box *vbox;
	AG_Box *hbox;
//	AG_Label *lbl;
	AG_Radio *rad;
	AG_Checkbox *cbox;
	AG_UCombo *ucom;

	int i;

	hbox_base = AG_BoxNewHoriz(win, AG_BOX_HFILL);
	vbox_left = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);
	vbox_right = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// FDD type
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "FDD Type (*)");

	strcpy(radFddTypeBase[0],"  No FDD");
	strcpy(radFddTypeBase[1],"  Use 3inch compact FDD");
	strcpy(radFddTypeBase[2],"  Use 5.25inch mini FDD");
	for(i=0; i<3; i++) {
		radFddType[i] = radFddTypeBase[i];
		if (i == config.fdd_type) {
			radFddTypeBase[i][0] = '>';
		}
	}
	radFddType[3]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radFddType, &paramtmp.fdd_type);
	AG_AddEvent(rad, "radio-changed", OnChangeFddType, "%Cp", this);

	// I/O Port address
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "I/O Port Address (*)");

	strcpy(chkIOPortBase[0],"  5.25inch FDC  $FF00 - $FF04");
	strcpy(chkIOPortBase[1],"  3inch FDC  $FF18 - $FF20");
	strcpy(chkIOPortBase[2],"  6voice PSG  $FF30 - $FF33");
	strcpy(chkIOPortBase[3],"  Ex LPT Port  $FF3C - $FF3F");
	strcpy(chkIOPortBase[4],"  Ex COM Port  $FF40 - $FF41");
	strcpy(chkIOPortBase[5],"  9voice PSG  $FF70 - $FF7F");
	strcpy(chkIOPortBase[6],"  KANJI ROM  $FF75 - $FF76");
	for(i=0; i<7; i++) {
		if (config.io_port & (1 << i)) {
			chkIOPortBase[i][0] = '>';
		}
		cbox = AG_CheckboxNewFlag(vbox, 0, chkIOPortBase[i], (Uint *)&paramtmp.io_port, (1 << i));
		AG_AddEvent(cbox, "checkbox-changed", OnChangeIOPort, "%Cp %i", this, i);
		chkIOPort[i] = cbox;
	}
	chkIOPort[7] = NULL;

	//
	AG_LabelNewS(vbox_left, AG_LABEL_HFILL, "* Need restart program or \"PowerOn\".");

	// Load wav file
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "Load wav file");

	AG_CheckboxNewInt(vbox, 0, "reverse wave", &paramtmp.wav_reverse);
	AG_CheckboxNewInt(vbox, 0, "half wave", &paramtmp.wav_half);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	cbox = AG_CheckboxNewInt(hbox, 0, "correct", &paramtmp.wav_correct);

	const char *radCorrect[] = {
		"cos wave",
		"sin wave",
		NULL
	};
	rad = AG_RadioNewInt(hbox, AG_RADIO_EXPAND, radCorrect, &paramtmp.wav_correct_type);

	AG_AddEvent(cbox, "checkbox-changed", OnChangeCorrect, "%Cp %Cp", this, rad);
	if (paramtmp.wav_correct == 0) {
		AG_WidgetDisable(rad);
	} else {
		AG_WidgetEnable(rad);
	}

	// Save wav file
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "Save wav file");

	const char *rate_list[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Sample rate");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (rate_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, rate_list[i]);
	}
	AG_UComboSizeHint(ucom, rate_list[i-1], i);
	AG_TlistSelectText(ucom->list, rate_list[paramtmp.wav_sample_rate]);
	AG_TlistSetChangedFn(ucom->list, OnChangeSampleRate, "%Cp", this);

	const char *bits_list[] = {
		"8",
		"16",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Sample bits");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (bits_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, bits_list[i]);
	}
	AG_UComboSizeHint(ucom, bits_list[i-1], i);
	AG_TlistSelectText(ucom->list, bits_list[paramtmp.wav_sample_bits]);
	AG_TlistSetChangedFn(ucom->list, OnChangeSampleBits, "%Cp", this);

	// CRTC
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "CRTC");

	const char *skew_list[] = {
		"0",
		"1",
		"2",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Disptmg skew");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (skew_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, skew_list[i]);
	}
	AG_UComboSizeHint(ucom, skew_list[i-1], i);
	AG_TlistSelectText(ucom->list, skew_list[config.disptmg_skew]);
	AG_TlistSetChangedFn(ucom->list, OnChangeDisptmgSkew, "%Cp", this);

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Curdisp skew");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (skew_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, skew_list[i]);
	}
	AG_UComboSizeHint(ucom, skew_list[i-1], i);
	AG_TlistSelectText(ucom->list, skew_list[config.curdisp_skew]);
	AG_TlistSetChangedFn(ucom->list, OnChangeCurdispSkew, "%Cp", this);


	// button
	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox_base, 0, "OK", OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox_base, 0, "Cancel", OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->Pause(true);
	AG_WindowShow(win);

	//
	change_fdd_type(paramtmp.fdd_type);
}

void AG_CONFIG_DLG::Update()
{
	config.fdd_type = param.fdd_type;

	switch(config.fdd_type) {
		case 0:
			emu->out_log(LOG_INFO,_T("Non FDD Mode"));
			break;
		case 1:
			emu->out_log(LOG_INFO,_T("3inch compact FDD Mode"));
			break;
		case 2:
			emu->out_log(LOG_INFO,_T("5.25inch mini FDD Mode"));
			break;
		default:
			break;
	}

	config.io_port = param.io_port;
}

int AG_CONFIG_DLG::GetParam(ConfigBoxParamId id)
{
	int data = 0;

	switch(id) {
		case FddType:
			data = param.fdd_type;
			break;
		case WavReverse:
			data = param.wav_reverse;
			break;
		case WavHalf:
			data = param.wav_half;
			break;
		case WavCorrect:
			data = param.wav_correct;
			break;
		case WavCorrectType:
			data = param.wav_correct_type;
			break;
		case WavSampleRate:
			data = param.wav_sample_rate;
			break;
		case WavSampleBits:
			data = param.wav_sample_bits;
			break;
		case IOPort:
			data = param.io_port;
			break;
	}

	return data;
}

void AG_CONFIG_DLG::change_fdd_type(int index)
{
	if (index == 0) {
		paramtmp.io_port = paramtmp.io_port & ~3;
		AG_WidgetEnable(chkIOPort[0]);
		AG_WidgetEnable(chkIOPort[1]);
	} else if (index == 1) {
		paramtmp.io_port = (paramtmp.io_port & ~1) | 2;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	} else if (index == 2) {
		paramtmp.io_port = (paramtmp.io_port & ~2) | 1;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	}
}

void AG_CONFIG_DLG::change_io_port(int index, int status)
{
	if (index == 0 && status & (1 << 0)) {
		paramtmp.io_port &= ~(1 << 1);
	} else if (index == 1 && status & (1 << 1)) {
		paramtmp.io_port &= ~(1 << 0);
	}
	if (index == 5 && status & (1 << 5)) {
		paramtmp.io_port &= ~(1 << 6);
	} else if (index == 6 && status & (1 << 6)) {
		paramtmp.io_port &= ~(1 << 5);
	}
}

/*
 * Event Handler (static)
 */
void AG_CONFIG_DLG::OnOk(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->param = dlg->paramtmp;

	config.disptmg_skew = dlg->param.disptmg_skew;
	config.curdisp_skew = dlg->param.curdisp_skew;

	save_config();

	AG_ObjectDetach(win);
	dlg->gui->Pause(false);
}

void AG_CONFIG_DLG::OnClose(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	AG_ObjectDetach(win);
	dlg->gui->Pause(false);
}

void AG_CONFIG_DLG::OnChangeCorrect(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Radio *rad = (AG_Radio *)AG_PTR(2);

	if (dlg->paramtmp.wav_correct == 0) {
		AG_WidgetDisable(rad);
	} else {
		AG_WidgetEnable(rad);
	}
}

void AG_CONFIG_DLG::OnChangeSampleRate(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp.wav_sample_rate = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeSampleBits(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp.wav_sample_bits = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFddType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = (int)AG_INT(2);

	dlg->change_fdd_type(index);
}

void AG_CONFIG_DLG::OnChangeIOPort(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = AG_INT(2);
	int status = AG_INT(3);

	dlg->change_io_port(index, status);
}

void AG_CONFIG_DLG::OnChangeDisptmgSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp.disptmg_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCurdispSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp.curdisp_skew = item->label;
	}
}
