/** @file ag_file_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.3.3

	@brief [ ag_file_dlg ]
*/

#include "ag_file_dlg.h"
#include "../emu.h"
#include "ag_gui_base.h"

extern void get_dir_and_basename(const char *, char *, char *);

AG_FILE_DLG::AG_FILE_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	datarec_opend = NO_DATA;
	for (int drv=0; drv<4; drv++) {
		disk_opend[drv] = NO_DATA;
	}

	flags = AG_FILEDLG_CLOSEWIN | AG_FILEDLG_EXPAND;
#ifdef AG_FILEDLG_MASK_EXT
	flags |= AG_FILEDLG_MASK_EXT;
#endif
}

AG_FILE_DLG::~AG_FILE_DLG()
{
}

/**
 * create load file dialog
 */
void AG_FILE_DLG::CreateLoad(FileDlgType type, const char *title, const char *ext, const char *dir, const char *file, void *attr)
{
	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL);

	AG_WindowSetCaptionS(win, title);

	AG_FileDlg *fd = AG_FileDlgNew(win, flags);

	if (dir != NULL && dir[0] != '\0') {
		AG_FileDlgSetDirectoryS(fd, dir);
	} else {
		AG_FileDlgSetDirectoryS(fd, emu->application_path());
	}
	if (file != NULL && file[0] != '\0') {
		AG_FileDlgSetFilenameS(fd, file);
	}
	if (ext != NULL) {
		AG_FileDlgAddType(fd, "Supported Files", ext, NULL, NULL);
	}
	AG_FileDlgAddType(fd, "All Files", "*.*", NULL, NULL);

	AG_FileDlgOkAction(fd, OnClickOkLoad, "%Cp %i %p", this, type, attr);
	AG_FileDlgCancelAction(fd, OnClose, "%Cp %Cp", this, fd);
	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, fd);

	AG_TextboxSizeHintPixels(fd->tbFile,fd->tbFile->ed->wPre,fd->tbFile->ed->hPre + 2);

	gui->Pause(true);
	AG_WindowShow(win);
}

void AG_FILE_DLG::CreateLoad(FileDlgType type, const char *title, const char *ext, const char *dir, const char *file)
{
	CreateLoad(type, title, ext, dir, file, NULL);
}

/**
 * create save file dialog
 */
void AG_FILE_DLG::CreateSave(FileDlgType type, const char *title, const char *ext, const char *dir, const char *file, void *attr)
{
	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL);

	AG_WindowSetCaptionS(win, title);

	AG_FileDlg *fd = AG_FileDlgNew(win, flags);

	if (dir != NULL && dir[0] != '\0') {
		AG_FileDlgSetDirectoryS(fd, dir);
	} else {
		AG_FileDlgSetDirectoryS(fd, emu->application_path());
	}
	if (file != NULL && file[0] != '\0') {
		AG_FileDlgSetFilenameS(fd, file);
	}
	if (ext != NULL) {
		AG_FileDlgAddType(fd, "Supported Files", ext, NULL, NULL);
	}
	AG_FileDlgAddType(fd, "All Files", "*.*", NULL, NULL);

	AG_FileDlgOkAction(fd, OnClickOkSave, "%Cp %i %p", this, type, attr);
	AG_FileDlgCancelAction(fd, OnClose, "%Cp %Cp", this, fd);
	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, fd);

	AG_TextboxSizeHintPixels(fd->tbFile,fd->tbFile->ed->wPre,fd->tbFile->ed->hPre + 2);

	gui->Pause(true);
	AG_WindowShow(win);
}

void AG_FILE_DLG::CreateSave(FileDlgType type, const char *title, const char *ext, const char *dir, const char *file)
{
	CreateSave(type, title, ext, dir, file, NULL);
}

/**
 * Create Overwrite dialog
 */
void AG_FILE_DLG::CreateOverwriteDlg(AG_FileDlg *fd, FileDlgType type, void *attr)
{
	AG_Window *win;
	AG_Button *btn;
	AG_HBox *hb;

	win = AG_WindowNew(AG_WINDOW_NORESIZE | AG_WINDOW_MODAL);
	AG_WindowSetCaptionS(win, "File already exists.");

	AG_WindowSetPosition(win, AG_WINDOW_CENTER, 0);
	AG_LabelNew(win, 0, "File %s exists. Overwrite?", fd->cfile);
	hb = AG_HBoxNew(win, AG_HBOX_HOMOGENOUS|AG_HBOX_HFILL);
	{
		AG_ButtonNewFn(hb, 0, "Yes",
		    OnClickOkOverwrite, "%Cp %Cp %Cp %i %p", this, win, fd, type, attr);
		btn = AG_ButtonNewFn(hb, 0, "No", AGWINDETACH(win));
		AG_WidgetFocus(btn);
	}

	AG_WindowShow(win);
}

//////////

void AG_FILE_DLG::Close(AG_FileDlg *fd)
{
	flags = fd->flags;

	AG_Window *win = AG_ParentWindow((void *)fd);
	if (win != NULL) {
		// close window
		AG_ObjectDetach(win);
	}
	gui->Pause(false);
}

void AG_FILE_DLG::LoadFile(AG_FileDlg *fd, FileDlgType type, void *attr)
{
	flags = fd->flags;

	switch(type) {
	case DATAREC:
		LoadDataRecFile(fd->cfile);
		break;
	case FLOPPY:
		OpenFloppyFile((int)(intptr_t)attr, fd->cfile, 0);
		break;
	case PRINTER:
		break;
	case STATUS:
		LoadStatusFile(fd->cfile);
		break;
	case AUTOKEY:
		LoadAutoKeyFile(fd->cfile);
		break;
	}

	AG_Window *win = AG_ParentWindow((void *)fd);
	if (win != NULL) {
		// close window
		AG_ObjectDetach((void *)win);
	}
	gui->Pause(false);
}

void AG_FILE_DLG::SaveFile(AG_FileDlg *fd, FileDlgType type, void *attr)
{
	flags = fd->flags;

	switch(type) {
	case DATAREC:
		SaveDataRecFile(fd->cfile);
		break;
	case FLOPPY:
		break;
	case PRINTER:
		SavePrinterFile((int)(intptr_t)attr, fd->cfile);
		break;
	case STATUS:
		SaveStatusFile(fd->cfile);
		break;
	case AUTOKEY:
		break;
	}

	AG_Window *win = AG_ParentWindow((void *)fd);
	if (win != NULL) {
		// close window
		AG_ObjectDetach((void *)win);
	}
	gui->Pause(false);
}

//////////

void AG_FILE_DLG::LoadDataRecFile(char *file_path)
{
	emu->out_debug(_T("tape image: %s"),file_path);
	if (emu->play_datarec(file_path)) {
		datarec_opend = LOAD_DATA;
		emu->out_debug(_T(" -> opend"));

		UPDATE_HISTORY(file_path, config.recent_datarec_path);
		copy_initial_path(config.initial_datarec_path, file_path);

	} else {
		emu->out_log(LOG_WARN, _T("Tape image couldn't be opend."));
		datarec_opend = NO_DATA;
	}
}

void AG_FILE_DLG::SaveDataRecFile(char *file_path)
{
	emu->out_debug(_T("tape image: %s"),file_path);
	if (emu->rec_datarec(file_path)) {
		datarec_opend = SAVE_DATA;
		emu->out_debug(_T(" -> opend"));

		UPDATE_HISTORY(file_path, config.recent_datarec_path);
		copy_initial_path(config.initial_datarec_path, file_path);

	} else {
		emu->out_log(LOG_ERROR, _T("Tape image couldn't be saved."));
	}
}

void AG_FILE_DLG::OpenFloppyFile(int drv, char *file_path, Uint32 flags)
{
	emu->out_debug(_T("floppy image: %s"),file_path);
	if (emu->open_disk(drv, file_path, 0, flags)) {
		disk_opend[drv] = OPEN_DATA;
		emu->out_debug(_T(" -> opend"));

		UPDATE_HISTORY(file_path, config.recent_disk_path[drv]);
		copy_initial_path(config.initial_disk_path, file_path);

	} else {
		emu->out_log(LOG_WARN, _T("Floppy image couldn't be opend."));
		disk_opend[drv] = NO_DATA;
	}
}

void AG_FILE_DLG::SavePrinterFile(int drv, char *file_path)
{
	emu->out_debug(_T("save print image: %s"),file_path);
	emu->save_printer(drv, file_path);
	copy_initial_path(config.initial_printer_path, file_path);
}

void AG_FILE_DLG::LoadStatusFile(char *file_path)
{
	emu->out_debug(_T("load status image: %s"),file_path);
	emu->load_state(file_path);
	copy_initial_path(config.initial_state_path, file_path);
}

void AG_FILE_DLG::SaveStatusFile(char *file_path)
{
	emu->out_debug(_T("save status image: %s"),file_path);
	emu->save_state(file_path);
	copy_initial_path(config.initial_state_path, file_path);
}

void AG_FILE_DLG::LoadAutoKeyFile(char *file_path)
{
	emu->out_debug(_T("load auto key file: %s"),file_path);
	emu->start_auto_key(file_path);
	copy_initial_path(config.initial_autokey_path, file_path);
}

void AG_FILE_DLG::copy_initial_path(char *dst, char *file_path)
{
	char dir_path[_MAX_PATH];

	get_dir_and_basename(file_path, dir_path, NULL);

	memset(dst, '\0', _MAX_PATH);
	strncpy(dst, dir_path, _MAX_PATH);
}

/*
 * Event Handler (static)
 */
void AG_FILE_DLG::OnClose(AG_Event *event)
{
	AG_FILE_DLG *dlg = (AG_FILE_DLG *)AG_PTR(1);
	AG_FileDlg *fd = (AG_FileDlg *)AG_PTR(2);

	dlg->Close(fd);
}

void AG_FILE_DLG::OnClickOkLoad(AG_Event *event)
{
	AG_FileDlg *fd = (AG_FileDlg *)AG_SELF();
	AG_FILE_DLG *dlg = (AG_FILE_DLG *)AG_PTR(1);
	FileDlgType type = (FileDlgType)AG_INT(2);
	void *attr = AG_PTR(3);

	if (AG_FileDlgCheckReadAccess(fd) == -1) {
		AG_TextMsgFromError();
		return;
	}
	dlg->LoadFile(fd, type, attr);
}

void AG_FILE_DLG::OnClickOkSave(AG_Event *event)
{
	AG_FileDlg *fd = (AG_FileDlg *)AG_SELF();
	AG_FILE_DLG *dlg = (AG_FILE_DLG *)AG_PTR(1);
	FileDlgType type = (FileDlgType)AG_INT(2);
	void *attr = AG_PTR(3);
	AG_FileInfo info;

	if (AG_GetFileInfo(fd->cfile, &info) == 0) {
		if (info.perms & AG_FILE_WRITEABLE) {
			dlg->CreateOverwriteDlg(fd, type, attr);
			return;
		}
	}
	dlg->SaveFile(fd, type, attr);
}

void AG_FILE_DLG::OnClickOkOverwrite(AG_Event *event)
{
	AG_FILE_DLG *dlg = (AG_FILE_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);
	AG_FileDlg *fd = (AG_FileDlg *)AG_PTR(3);
	FileDlgType type = (FileDlgType)AG_INT(4);
	void *attr = AG_PTR(5);

	AG_ObjectDetach(win);

	dlg->SaveFile(fd, type, attr);
}
