/** @file ag_file_dlg.h

	Skelton for retropc emulator
	SDL + Agar edition

	@author Sasaji
	@date   2012.03.01

	@brief [ ag_file_dlg ]
*/

#ifndef _AG_FILE_DLG_H_
#define _AG_FILE_DLG_H_

#include "ag_dlg.h"

#define UPDATE_HISTORY(path, recent) { \
	int no = 7; \
	for(int i = 0; i < 8; i++) { \
		if(strcmp(recent[i], path) == 0) { \
			no = i; \
			break; \
		} \
	} \
	for(int i = no; i > 0; i--) { \
		strcpy(recent[i], recent[i - 1]); \
	} \
	strcpy(recent[0], path); \
}

/**
	@brief file dialog
*/
class AG_FILE_DLG : public AG_DLG {
	friend class AG_GUI_BASE;

private:
	enum OpendType {
		NO_DATA,
		OPEN_DATA,
		LOAD_DATA,
		SAVE_DATA
	};

	OpendType datarec_opend;
	OpendType disk_opend[4];

	Uint32 flags;

	void copy_initial_path(char *, char *);

	// file dialog event handler
	static void OnClose(AG_Event *);
	static void OnClickOkLoad(AG_Event *);
	static void OnClickOkSave(AG_Event *);
	static void OnClickOkOverwrite(AG_Event *);

public:
	AG_FILE_DLG(EMU *, AG_GUI_BASE *);
	~AG_FILE_DLG();

	enum FileDlgType {
		DATAREC,
		FLOPPY,
		PRINTER,
		STATUS,
		AUTOKEY
	};
	void CreateLoad(FileDlgType, const char *, const char *, const char *, const char *, void *);
	void CreateLoad(FileDlgType, const char *, const char *, const char *, const char *);
	void CreateSave(FileDlgType, const char *, const char *, const char *, const char *, void *);
	void CreateSave(FileDlgType, const char *, const char *, const char *, const char *);
    void CreateOverwriteDlg(AG_FileDlg *, FileDlgType, void *);

	void Close(AG_FileDlg *);
	void LoadFile(AG_FileDlg *, FileDlgType, void *);
	void SaveFile(AG_FileDlg *, FileDlgType, void *);

	void LoadDataRecFile(char *);
	void SaveDataRecFile(char *);
	void OpenFloppyFile(int, char *, Uint32);
	void SavePrinterFile(int, char *);
	void LoadStatusFile(char *);
	void SaveStatusFile(char *);
	void LoadAutoKeyFile(char *);

};

#endif /* _AG_FILE_DLG_H_ */
