/** @file ag_gui_base.h

	Skelton for retropc emulator
	SDL + Agar edition

	@author Sasaji
	@date   2012.03.01

	@brief [ ag_gui_base ]
*/

#ifndef _AG_GUI_BASE_H_
#define _AG_GUI_BASE_H_

#include <agar/config/have_sdl.h>
#include <agar/core.h>
#include <agar/gui.h>
#include <agar/gui/cursors.h>
#include <agar/gui/sdl.h>
#include <SDL.h>
#include "../common.h"
#include "../version.h"
#include "../emu.h"

class EMU;
class AG_DLG;
class AG_FILE_DLG;
class AG_VOLUME_DLG;

#define AG_GUI_MENU_CHECK(mitem, val) { \
	if ((mitem) && (mitem)->text) { \
		if (val) { \
			*(mitem)->text = '*'; \
		} else { \
			*(mitem)->text = ' '; \
		} \
	} \
}
#define AG_GUI_MENU_ENABLE(mitem, val) { \
	if (mitem) { \
		if (val) { \
			mitem->state = 1; \
		} else { \
			mitem->state = 0; \
		} \
	} \
}

/**
	@brief gui base class
*/
class AG_GUI_BASE {
private:
	int exit_event;

	virtual void set_menu_item(AG_Menu *);
	virtual void set_global_keys();

	void event_proc(AG_DriverEvent *);
	void gui_focus_out(int, int);

	void translate_keycode(SDL_KeyboardEvent *);


protected:
	EMU       *emu;

	typedef struct st_ag_drvs {
		AG_DriverClass	*ops;
		AG_DriverSw		*sw;
		AG_Window		*win;
		AG_Menu			*menu;
	} ag_driver_t;

	ag_driver_t drivers[2];
	ag_driver_t *now_driver;

	// menu event handler
	static void OnSelectExit(AG_Event *);

	static void OnSelectReset(AG_Event *);
	static void OnSelectSpecialReset(AG_Event *);
	static void OnSelectWarmReset(AG_Event *);

	static void OnSelectDipswitch(AG_Event *);
	static void OnUpdateDipswitch(AG_Event *);

	static void OnSelectPause(AG_Event *);
	static void OnUpdatePause(AG_Event *);

	static void OnSelectCPUPower(AG_Event *);
	static void OnUpdateCPUPower(AG_Event *);

	static void OnSelectStartAutoKey(AG_Event *);
	static void OnUpdateStartAutoKey(AG_Event *);
	static void OnSelectStopAutoKey(AG_Event *);

	static void OnSelectLoadDataRec(AG_Event *);
	static void OnUpdateLoadDataRec(AG_Event *);
	static void OnSelectSaveDataRec(AG_Event *);
	static void OnUpdateSaveDataRec(AG_Event *);
	static void OnSelectCloseDataRec(AG_Event *);
	static void OnSelectRecentDataRec(AG_Event *);

	static void OnSelectOpenFloppyDisk(AG_Event *);
	static void OnUpdateOpenFloppyDisk(AG_Event *);
	static void OnSelectCloseFloppyDisk(AG_Event *);
	static void OnSelectRecentFloppyDisk(AG_Event *);

	static void OnSelectFrameRate(AG_Event *);
	static void OnUpdateFrameRate(AG_Event *);
	static void OnSelectScreenRecordSize(AG_Event *);
	static void OnUpdateScreenRecordSize(AG_Event *);
	static void OnSelectScreenCapture(AG_Event *);
	static void OnSelectWindowMode(AG_Event *);
	static void OnUpdateWindowMode(AG_Event *);
	static void OnSelectScreenMode(AG_Event *);
	static void OnUpdateScreenMode(AG_Event *);
	static void OnSelectScanLine(AG_Event *);
	static void OnUpdateScanLine(AG_Event *);

	static void OnSelectSoundVolume(AG_Event *);
	static void OnSelectSoundStartRecord(AG_Event *);
	static void OnUpdateSoundStartRecord(AG_Event *);
	static void OnSelectSoundStopRecord(AG_Event *);
	static void OnUpdateSoundStopRecord(AG_Event *);
	static void OnSelectSoundRate(AG_Event *);
	static void OnUpdateSoundRate(AG_Event *);
	static void OnSelectSoundLatency(AG_Event *);
	static void OnUpdateSoundLatency(AG_Event *);

	static void OnSelectSavePrinter(AG_Event *);
	static void OnUpdateSavePrinter(AG_Event *);
	static void OnSelectClearPrinter(AG_Event *);
	static void OnUpdateClearPrinter(AG_Event *);

	static void OnUpdateRecent(AG_Event *);

	static void OnSelectAbout(AG_Event *);

	static void OnCloseDialog(AG_Event *);

public:
	AG_GUI_BASE(EMU *);
	~AG_GUI_BASE();

	// event data from SDL
	SDL_Event sdl_event;

	// file dialog
	AG_FILE_DLG *filebox;
	// volume dialog
	AG_VOLUME_DLG *volumebox;


	virtual int InitCore();
	int InitVideoSDL(SDL_Surface *, int, int);
	int ResizeDisplay(int width, int height);
	int CreateMenu();
	int CreateGlobalKeys();
	int EventLoop();
	int GetNextEvent(void *, AG_DriverEvent *);
	int ProcessEvent(SDL_Event *);
	int MixGuiScreen();
	int ProcessTimeout();
	void Show(bool val); 
	void Pause(bool val);

	int ConfigLoad();
	int ConfigSave();

	virtual AG_DLG *GetDlgPtr(int);
	virtual bool ExecuteGlobalKeys(int key, Uint32 mod);

	// event operation
	virtual void Exit(void);
	virtual void Reset(void);
	virtual void SpecialReset(void);
	virtual void WarmReset(void);
	virtual void Dipswitch(int bit);
	virtual void Pause(void);
	virtual void CPUPower(int num);
	virtual void ChangeWindowMode(int num);
	virtual void ChangeScreenMode(int num);
	virtual void ToggleScanLine(void);
	virtual void CreateVolumeBox(void);

};

#endif /* _AG_GUI_BASE_H_ */
