/** @file ag_gui_config.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.3.2

	@brief [ ag_gui_base ]
*/

#include "ag_gui_base.h"
#include "../emu.h"
#include "../config.h"

#define FONT_MAX 3

int AG_GUI_BASE::ConfigLoad()
{
	char fpath[AG_PATHNAME_MAX];

	struct fdata_st {
		char name[AG_PATHNAME_MAX];
		int  size;
		Uint32 flags;
	} fdata[FONT_MAX];

	memset(fdata, 0, sizeof(fdata));

#if defined(_WIN32)
#ifdef AG_PATHSEPMULTI
	AG_PrtString(agConfig, "font-path", "%s\\res;%s;%s\\fonts"
										,emu->application_path(),emu->application_path(),SDL_getenv("SystemRoot"));
#else
	AG_PrtString(agConfig, "font-path", "%s\\res:%s:%s\\fonts"
										,emu->application_path(),emu->application_path(),SDL_getenv("SystemRoot"));
#endif
	strcpy(fdata[0].name, config.menu_fontname);  fdata[0].size = 12;
	AG_SetInt(agConfig, "font.size", 12);

#elif defined(linux)
	AG_PrtString(agConfig, "font-path", "%s/res:%s:"
										"%s/.fonts:"
										"/usr/share/fonts/truetype:"
										"/usr/X11R6/lib/X11/fonts/TTF"
										,emu->application_path(),emu->application_path(),getenv("HOME"));
	strcpy(fdata[0].name, config.menu_fontname);  fdata[0].size = 12;
	AG_SetInt(agConfig, "font.size", 12);

#elif defined(__APPLE__) && defined(__MACH__)
	AG_PrtString(agConfig, "font-path", "%s/res:%s:"
										"%s/Library/Fonts:/Library/Fonts:"
										"/System/Library/Fonts"
										,emu->application_path(),emu->application_path(),getenv("HOME"));
	strcpy(fdata[0].name, config.menu_fontname);  fdata[0].size = 12;
	AG_SetInt(agConfig, "font.size", 12);

#elif defined(__FreeBSD__)
	AG_PrtString(agConfig, "font-path", "%s/res:%s:"
										"%s/.fonts:"
										"/usr/share/fonts/TTF:"
										"/usr/local/lib/X11/fonts/TTF"
										,emu->application_path(),emu->application_path(),getenv("HOME"));
	strcpy(fdata[0].name, config.menu_fontname);  fdata[0].size = 12;
	AG_SetInt(agConfig, "font.size", 12);

#endif

	// search specified fonts
	for(int i=0; i< FONT_MAX; i++) {
		if (fdata[i].name == NULL || fdata[i].name[0] == '\0') {
			continue;
		}
		if (AG_ConfigFile("font-path", fdata[i].name, NULL, fpath, AG_PATHNAME_MAX - 1) == 0) {
			// found
			emu->out_log(LOG_INFO, _T("AG_MENU: use font: %s"), fpath);
			AG_PrtString(agConfig, "font.face", "%s", fdata[i].name);
			AG_SetInt(agConfig, "font.size", fdata[i].size);
			AG_SetUint32(agConfig, "font.flags", fdata[i].flags);
			break;
		} else {
			// not found
			emu->out_log(LOG_WARN, _T("AG_ConfigFile: %s"), AG_GetError());
		}
	}
	return 0;
}

int AG_GUI_BASE::ConfigSave()
{
	return 0;
}
