/** @file ag_keybind_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.4.8

	@brief [ ag_keybind_dlg ]
*/

#include "ag_keybind_dlg.h"
#include "ag_gui_base.h"
#include "../emu.h"


AG_KEYBIND_DLG::AG_KEYBIND_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(table, 0, sizeof(table));
	memset(row2idx_map, 0, sizeof(row2idx_map));

	tm1 = 0;

	joy_stat = emu->joy_buffer();
}

AG_KEYBIND_DLG::~AG_KEYBIND_DLG()
{
}

/*
 * create keybind dialog
 */
void AG_KEYBIND_DLG::Create()
{
//	configdlgparamtmp = configdlgparam;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE | AG_WINDOW_MODKEYEVENTS);
	AG_WindowSetCaptionS(win, "Keybind");

	AG_Notebook *nb;
//	AG_NotebookTab *ntab;
	AG_Table *tbl;
	AG_Box *hbox;
	AG_Box *vbox;

	char label[3][100];
	char str[40];
	int tab;
	int i;
	Uint32 code;

	selected_tab = 0;
	memset(selected, 0, sizeof(selected));

	nb = AG_NotebookNew(win, AG_NOTEBOOK_HFILL);

	for(tab=0; tab<MAX_TAB_NUM; tab++) {
		int rows = vmkey_map_size[tab];
		int cols = vkkey_defmap_cols[tab];

		if (tab == 1) tabs[tab] = AG_NotebookAddTab(nb, "joypad", AG_BOX_HORIZ);
		else tabs[tab] = AG_NotebookAddTab(nb, "keyboard", AG_BOX_HORIZ);

		// table
		vbox = AG_BoxNewVert(tabs[tab], AG_BOX_VFILL);
		if (tab == 1) {
			tbl = AG_TableNewPolled(vbox, AG_TABLE_EXPAND | AG_TABLE_NOAUTOSORT, OnUpdate, "%Cp", this);
			AG_TableSetPollInterval(tbl, 250);
		} else {
			tbl = AG_TableNew(vbox, AG_TABLE_EXPAND | AG_TABLE_NOAUTOSORT);
		}
		tbl->wid.flags |= AG_WIDGET_CATCH_TAB;
		tbls[tab] = tbl;
		AG_TableSetSelectionMode(tbl, AG_TABLE_SEL_CELLS);
		AG_TableSetColMin(tbl, 80);
		AG_TableSetDefaultColWidth(tbl, 120);
		if (tab == 1) {
			AG_TableAddCol(tbl, "level3", "100px", NULL);
			AG_TableAddCol(tbl, "joypad1", "120px", NULL);
			AG_TableAddCol(tbl, "joypad2", "120px", NULL);
		} else {
			AG_TableAddCol(tbl, "level3", "100px", NULL);
			AG_TableAddCol(tbl, "bind1", "120px", NULL);
			AG_TableAddCol(tbl, "bind2", "120px", NULL);
		}

		AG_TableBegin(tbl);
		int validrow = 0;
		for(int row=0; row<rows; row++) {
			if (table[tab][row].vm_keycode >= vkkey_defmap_rows[tab]) {
				table[tab][row].enabled = false;
			}

			if (table[tab][row].enabled) {
				get_vmkeylabel(row, label[0]);

				for(int col=0; col<cols; col++) {
					code = *(vkkey_map[tab] + table[tab][row].vm_keycode * cols + col);
					if (tab == 1) SetVkJoyCode(&table[tab][row].cols[col], code, label[col + 1]);
					else SetVkKeyCode(&table[tab][row].cols[col], code, label[col + 1]);
				}
				AG_TableAddRow(tbl, "%s:%s:%s",label[0],label[1],label[2]);
				table[tab][row].row = validrow;
				row2idx_map[tab][validrow] = row;
				validrow++;
			} else {
				table[tab][row].row = -1;
			}
		}
		AG_TableAddRow(tbl, "%s:%s:%s","","","");
		AG_TableEnd(tbl);

		AG_TableSetCellClickFn(tbl, OnClickCell, "%Cp", this);
		AG_SetEvent(tbl, "key-down", OnKeyDown, "%Cp", this);

		// checkbox for joypad
		AG_LabelNew(vbox, 0, " ");
		if (tab == 1) {
			combi1 = *(vkkey_map[tab] + (vkkey_defmap_rows[tab] - 1) * cols);
			AG_CheckboxNewInt(vbox, 0, "Recognize as another key when pressed two buttons", &combi1);
		} else {
			AG_LabelNew(vbox, 0, " ");
		}

		// right button
		vbox = AG_BoxNewVert(tabs[tab],AG_BOX_EXPAND);
		AG_ButtonNewFn(vbox, AG_BUTTON_HFILL, "load default", OnLoadDefault, "%Cp", this);
		AG_LabelNew(vbox, 0, " ");
		for(i=0; i<4; i++) {
			sprintf(str, "load preset %d", i+1);
			AG_ButtonNewFn(vbox, AG_BUTTON_HFILL, str, OnLoadPreset, "%Cp %i", this, i);
		}
		AG_LabelNew(vbox, 0, " ");
		for(i=0; i<4; i++) {
			sprintf(str, "save preset %d", i+1);
			AG_ButtonNewFn(vbox, AG_BUTTON_HFILL, str, OnSavePreset, "%Cp %i", this, i);
		}

		AG_LabelNew(vbox, 0, " ");
	}

	// add event
	AG_AddEvent(nb, "mouse-button-down", OnClickTab, "%Cp", this);

	// bottom button
	hbox = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox, 0, "OK", OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox, 0, "Cancel", OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->Pause(true);
	AG_WindowShow(win);

	// bug fix for agar
	for(i=1; i<3; i++) {
		tbls[0]->cols[i].x = tbls[0]->cols[i-1].x + tbls[0]->cols[i-1].w + 1;
	}
	for(tab=1; tab<MAX_TAB_NUM; tab++) {
		for(i=0; i<3; i++) {
			tbls[tab]->cols[i].x = tbls[0]->cols[i].x;
			tbls[tab]->cols[i].w = tbls[0]->cols[i].w;
		}
	}
}

void AG_KEYBIND_DLG::Update()
{
	int tab = selected_tab;
	AG_Table *tbl = tbls[tab];
	char label[3][100];

	AG_TableBegin(tbl);
	for(int row=0; row<vmkey_map_size[tab]; row++) {
		if (table[tab][row].enabled) {
			get_vmkeylabel(row, label[0]);

			for(int col=0; col<vkkey_defmap_cols[tab]; col++) {
				if (tab == 1) get_vkjoylabel(table[tab][row].cols[col].vk_keycode, label[col + 1]);
				else get_vkkeylabel(table[tab][row].cols[col].vk_keycode, label[col + 1]);
			}
			AG_TableAddRow(tbl, "%s:%s:%s",label[0],label[1],label[2]);

		}
	}
	AG_TableAddRow(tbl, "%s:%s:%s","","","");
	AG_TableEnd(tbl);

	AG_TableDeselectAllRows(tbl);
	AG_TableSelectCell(tbl, selected[tab].row, selected[tab].col);
}

void AG_KEYBIND_DLG::SetVmKeyMap(int tab_num, Uint16 *vmKeyMap, int size)
{
	if (tab_num < 0 || tab_num >= MAX_TAB_NUM) return;

	for(int i=0; i<KBCTRL_MAX_LINES && i<size; i++) {
//		vmkey_map[tab_num][i] = vmKeyMap[i];
		SetVmKeyCode(tab_num, i, vmKeyMap[i]);
	}
	vmkey_map_size[tab_num] = (size > KBCTRL_MAX_LINES) ? KBCTRL_MAX_LINES : size;
}

void AG_KEYBIND_DLG::SetVmKey(int tab_num, int idx, Uint16 code)
{
	if (tab_num < 0 || tab_num >= MAX_TAB_NUM) return;

//	vmkey_map[tab_num][idx] = code;
	SetVmKeyCode(tab_num, idx, code);
}

bool AG_KEYBIND_DLG::SetVmKeyCode(int tab_num, int idx, Uint16 code)
{
	if (idx < 0 || idx >= KBCTRL_MAX_LINES) {
		return false;
	}
	if (code >= 0xff) {
		table[tab_num][idx].enabled = false;
	} else {
		table[tab_num][idx].enabled = true;
	}

	table[tab_num][idx].vm_keycode = code;

	return true;
}

void AG_KEYBIND_DLG::SetVkKeyMap(int tab_num, Uint32 *vkKeyMap)
{
	if (tab_num < 0 || tab_num >= MAX_TAB_NUM) return;

	vkkey_map[tab_num] = vkKeyMap;
}

void AG_KEYBIND_DLG::SetVkKeyDefMap(int tab_num, Uint32 *vkKeyDefMap, int rows, int cols)
{
	if (tab_num < 0 || tab_num >= MAX_TAB_NUM) return;

	vkkey_defmap[tab_num] = vkKeyDefMap;
	vkkey_defmap_rows[tab_num] = rows;
	vkkey_defmap_cols[tab_num] = cols;
}

void AG_KEYBIND_DLG::SetVkKeyPresetMap(int tab_num, Uint32 *vkKeyMap, int idx)
{
	if (tab_num < 0 || tab_num >= MAX_TAB_NUM) return;

	if (idx < 4) {
		vkkey_preset_map[tab_num][idx] = vkKeyMap;
	}
}

bool AG_KEYBIND_DLG::SetVkKeyCode(codecols_t *obj, Uint32 code, char *label)
{
	Uint32 new_code = 0;

	new_code = translate_vkkey(code);
	if (label != NULL) get_vkkeylabel(new_code, label);
	obj->vk_prev_keycode = obj->vk_keycode;
	obj->vk_keycode = new_code;

	return true;
}

bool AG_KEYBIND_DLG::SetVkJoyCode(codecols_t *obj, Uint32 code, char *label)
{
	bool rc = true;
	if (label != NULL) {
		rc = get_vkjoylabel(code, label);
	}
	if (rc == true) {
		obj->vk_prev_keycode = obj->vk_keycode;
		obj->vk_keycode = code;
	}
	return true;
}

void AG_KEYBIND_DLG::SetTitleLabel(const _TCHAR *vm, const _TCHAR *vk, const _TCHAR *vj)
{
}

bool AG_KEYBIND_DLG::get_vmkeylabel(int code, char *label)
{
	bool known_key = true;

	switch(code) {
		case 0x00:
			strcpy(label, "");
			break;
		case 0x01:
			strcpy(label, "break");
			break;
		case 0x02:
			strcpy(label, "reset");
			break;
		case 0x03:
			strcpy(label, "mode");
			break;
		case 0x04:
			strcpy(label, "(Pause)");
			break;
		case 0x05:
			strcpy(label, "(Alt)");
			break;
		case 0x06:
			strcpy(label, "ins/del");
			break;
		case 0x07:
			strcpy(label, "num ins/del");
			break;
		case 0x08:
			strcpy(label, "bs");
			break;
		case 0x09:
			strcpy(label, "tab");
			break;
		case 0x0c:
			strcpy(label, "home/cls");
			break;
		case 0x0d:
			strcpy(label, "return");
			break;
		case 0x0e:
			strcpy(label, "enter");
			break;
		case 0x0f:
			strcpy(label, "help");
			break;

		case 0x10:
			strcpy(label, "shift");
			break;
		case 0x11:
			strcpy(label, "ctrl");
			break;
		case 0x14:
			strcpy(label, "caps");
			break;
		case 0x15:
			strcpy(label, "kana");
			break;
		case 0x16:
			strcpy(label, "graph");
			break;
		case 0x17:
			strcpy(label, "henkan");
			break;
		case 0x18:
			strcpy(label, "muhenkan");
			break;
		case 0x19:
			strcpy(label, "command");
			break;
		case 0x1a:
			strcpy(label, "option");
			break;

		case 0x1b:
			strcpy(label, "esc");
			break;
		case 0x1c:
			strcpy(label, "right");
			break;
		case 0x1d:
			strcpy(label, "left");
			break;
		case 0x1e:
			strcpy(label, "up");
			break;
		case 0x1f:
			strcpy(label, "down");
			break;
		case 0x20:
			strcpy(label, "space");
			break;
		// 0x21 - 0x28  button1 - 8

		// 0x29 - 0x2f  unknown

		// 0x30 - 0x39  0 - 9
		case 0x3a:
			strcpy(label, ": *");
			break;
		case 0x3b:
			strcpy(label, "; +");
			break;
		case 0x3c:
			strcpy(label, ", <");
			break;
		case 0x3d:
			strcpy(label, "- =");
			break;
		case 0x3e:
			strcpy(label, ". >");
			break;
		case 0x3f:
			strcpy(label, "/ ?");
			break;
		case 0x40:
			strcpy(label, "@");
			break;
		// 0x41 - 0x5a  A - Z
		case 0x5b:
			strcpy(label, "[");
			break;
		case 0x5c:
			strcpy(label, "\\");
			break;
		case 0x5d:
			strcpy(label, "]");
			break;
		case 0x5e:
			strcpy(label, "^");
			break;
		case 0x5f:
			strcpy(label, "_");
			break;
		// 0x60 - 0x69  num 0 - num 9
		case 0x6a:
			strcpy(label, "num *");
			break;
		case 0x6b:
			strcpy(label, "num +");
			break;
		case 0x6c:
			strcpy(label, "num ?");
			break;
		case 0x6d:
			strcpy(label, "num -");
			break;
		case 0x6e:
			strcpy(label, "num .");
			break;
		case 0x6f:
			strcpy(label, "num /");
			break;
		// 0x70 - 0x7f  PF1 - PF16

		default:
			if (0x30 <= code && code <= 0x39) {
				// 0 - 9
				sprintf(label,"%c",code);
			} else if (0x41 <= code && code <= 0x5a) {
				// A - Z
				sprintf(label,"%c",code);
			} else if (0x60 <= code && code <= 0x69) {
				// num 0 - num 9
				sprintf(label,"num %c",code - 0x30);
			} else if (0x70 <= code && code <= 0x7f) {
				// PF1 - PF16
				sprintf(label,"PF%d",code - 0x6f);
			} else {
				sprintf(label,"0x%02x",code);
				known_key = false;

			}
			break;
	}

	return known_key;
}

Uint32 AG_KEYBIND_DLG::translate_vkkey(Uint32 code) {
//	UINT32 new_code = code;

//	emu->translate_sdl_keysym();
	return code;
}

bool AG_KEYBIND_DLG::get_vkkeylabel(Uint32 code, char *label) {
	bool known_key = true;

	switch(code) {
		case 0:
			strcpy(label, "");
			break;
		case SDLK_BACKSPACE:
			strcpy(label, "bs");
			break;
		case SDLK_TAB:
			strcpy(label, "tab");
			break;
		case SDLK_CLEAR:
			strcpy(label, "clear");
			break;
		case SDLK_RETURN:
			strcpy(label, "enter");
			break;
		case SDLK_PAUSE:
			strcpy(label, "pause");
			break;

		case SDLK_ESCAPE:
			strcpy(label, "esc");
			break;
		case SDLK_SPACE:
			strcpy(label, "space");
			break;

		case SDLK_COMMA:
			strcpy(label, ", <");	// JP
			break;
		case SDLK_MINUS:
			strcpy(label, "- =");	// JP
			break;
		case SDLK_PERIOD:
			strcpy(label, ". >");	// JP
			break;
		case SDLK_SLASH:
			strcpy(label, "/ ?");	// JP
			break;

		case SDLK_COLON:
			strcpy(label, ": *");	// JP
			break;
		case SDLK_SEMICOLON:
			strcpy(label, "; +");	// JP
			break;
		case SDLK_AT:
			strcpy(label, "@");	// JP
			break;
		case SDLK_LEFTBRACKET:
			strcpy(label, "[");	// JP
			break;
		case SDLK_BACKSLASH:
			strcpy(label, "\\");	// JP
			break;
		case SDLK_RIGHTBRACKET:
			strcpy(label, "]");	// JP
			break;
		case SDLK_CARET:
			strcpy(label, "^");	// JP
			break;
		case SDLK_UNDERSCORE:
			strcpy(label, "_");	// JP
			break;
		case SDLK_BACKQUOTE:
			strcpy(label, "kanji");	// JP
			break;

		case SDLK_DELETE:
			strcpy(label, "delete");
			break;

		case SDLK_WORLD_0:	// SDLK_WORLD_0
			strcpy(label, "henkan");
			break;
		case SDLK_WORLD_1:	// SDLK_WORLD_1
			strcpy(label, "muhenkan");
			break;
		case SDLK_WORLD_2:	// SDLK_WORLD_2
			strcpy(label, "katakana");
			break;

		case SDLK_KP_PERIOD:
			strcpy(label, "num .");
			break;
		case SDLK_KP_DIVIDE:
			strcpy(label, "num /");
			break;
		case SDLK_KP_MULTIPLY:
			strcpy(label, "num *");
			break;
		case SDLK_KP_MINUS:
			strcpy(label, "num -");
			break;
		case SDLK_KP_PLUS:
			strcpy(label, "num +");
			break;
		case SDLK_KP_ENTER:
			strcpy(label, "num enter");
			break;
		case SDLK_KP_EQUALS:
			strcpy(label, "num =");
			break;

		case SDLK_UP:
			strcpy(label, "up");
			break;
		case SDLK_DOWN:
			strcpy(label, "down");
			break;
		case SDLK_RIGHT:
			strcpy(label, "right");
			break;
		case SDLK_LEFT:
			strcpy(label, "left");
			break;
		case SDLK_INSERT:
			strcpy(label, "insert");
			break;
		case SDLK_HOME:
			strcpy(label, "home");
			break;
		case SDLK_END:
			strcpy(label, "end");
			break;
		case SDLK_PAGEUP:
			strcpy(label, "page up");
			break;
		case SDLK_PAGEDOWN:
			strcpy(label, "page down");
			break;

		case SDLK_NUMLOCK:
			strcpy(label, "numlock");
			break;
		case SDLK_CAPSLOCK:
			strcpy(label, "caps");
			break;
		case SDLK_SCROLLOCK:
			strcpy(label, "scroll lock");
			break;
		case SDLK_RSHIFT:
			strcpy(label, "rshift");
			break;
		case SDLK_LSHIFT:
			strcpy(label, "lshift");
			break;
		case SDLK_RCTRL:
			strcpy(label, "rctrl");
			break;
		case SDLK_LCTRL:
			strcpy(label, "lctrl");
			break;
		case SDLK_RALT:
			strcpy(label, "ralt");
			break;
		case SDLK_LALT:
			strcpy(label, "lalt");
			break;
		case SDLK_RMETA:
			strcpy(label, "rmeta");
			break;
		case SDLK_LMETA:
			strcpy(label, "lmeta");
			break;
		case SDLK_RSUPER:
			strcpy(label, "rsuper");
			break;
		case SDLK_LSUPER:
			strcpy(label, "lsuper");
			break;
		case SDLK_MODE:
			strcpy(label, "mode");
			break;
		case SDLK_COMPOSE:
			strcpy(label, "compose");
			break;
		case SDLK_HELP:
			strcpy(label, "help");
			break;
		case SDLK_PRINT:
			strcpy(label, "print");
			break;
		case SDLK_SYSREQ:
			strcpy(label, "sys req");
			break;
		case SDLK_BREAK:
			strcpy(label, "break");
			break;
		case SDLK_MENU:
			strcpy(label, "menu");
			break;
		default:
			if (0x30 <= code && code <= 0x39) {
				// 0 - 9
				sprintf(label,"%c",code);
			} else if (0x61 <= code && code <= 0x7a) {
				// A - Z
				sprintf(label,"%c",code - 0x20);
			} else if (0x100 <= code && code <= 0x109) {
				// num 0 - num 9
				sprintf(label,"num %c",code - 0xd0);
			} else if (0x11a <= code && code <= 0x128) {
				// F1 - F15
				sprintf(label,"F%d",code - 0x119);
			} else {
				sprintf(label,"0x%02x",code);
				known_key = false;
			}
			break;
	}

	return known_key;
}

bool AG_KEYBIND_DLG::get_vkjoylabel(Uint32 code, char *label) {
	bool known_key = true;
	char numstr[8];

	switch(code) {
		case 0x00:
			strcpy(label, "");
			break;
		case 0x01:
			strcpy(label, "up");
			break;
		case 0x02:
			strcpy(label, "down");
			break;
		case 0x04:
			strcpy(label, "left");
			break;
		case 0x05:
			strcpy(label, "up+left");
			break;
		case 0x06:
			strcpy(label, "down+left");
			break;
		case 0x08:
			strcpy(label, "right");
			break;
		case 0x09:
			strcpy(label, "up+right");
			break;
		case 0x0a:
			strcpy(label, "down+right");
			break;
		default:
			// button
			if (code & 0xfffffff0) {
				strcpy(label, "button");
				for(int i=0; i<28; i++) {
					if (code & (0x10 << i)) {
						sprintf(numstr, " %d", i+1);
						strcat(label, numstr);
					}
				}
			} else {
				strcpy(label, "");
				known_key = false;
			}
			break;
	}

	return known_key;
}

void AG_KEYBIND_DLG::click_cell(AG_Table *tbl, int row, int col)
{
	int tab = selected_tab;
	int idx = row2idx_map[tab][row];
	codecols_t *obj;

	if (selected[tab].cell == AG_TableGetCell(tbl, row, col)) {
		// double click ?
		Uint32 tm2 = AG_GetTicks();
		if (tm2 - tm1 < 500) {
			obj = &table[tab][idx].cols[col-1];
			if (tab == 1) SetVkJoyCode(obj, 0, NULL);
			else SetVkKeyCode(obj, 0, NULL);
			Update();
		}
	}

	AG_TableDeselectRow(tbl, row);
	if (row + 1 < tbl->m) {
		if (col < 1) col = 1;
		AG_TableSelectCell(tbl, row, col);

		selected[tab].cell = AG_TableGetCell(tbl, row, col);
		selected[tab].row = row;
		selected[tab].col = col;
	}
	tm1 = AG_GetTicks();
}

void AG_KEYBIND_DLG::load_data(int num)
{
	int tab = selected_tab;
	int rows = vmkey_map_size[tab];
	int cols = vkkey_defmap_cols[tab];
	Uint32 *dst;
	Uint32 code;

	if (0 <= num && num <= MAX_PRESET) {
		dst = vkkey_preset_map[tab][num];
	} else {
		dst = vkkey_defmap[tab];
	}

	for(int row=0; row<rows; row++) {
		if (table[tab][row].enabled) {
			for(int col=0; col<cols; col++) {
				code = dst[table[tab][row].vm_keycode * cols + col];
				if (tab == 1) SetVkJoyCode(&table[tab][row].cols[col], code, NULL);
				else SetVkKeyCode(&table[tab][row].cols[col], code, NULL);
			}
		}
	}
	if (tab == 1) {
		combi1 = dst[(vkkey_defmap_rows[tab] - 1) * cols];
	}
	Update();
}

void AG_KEYBIND_DLG::save_data(int num)
{
	int tab = selected_tab;
	int rows = vmkey_map_size[tab];
	int cols = vkkey_defmap_cols[tab];
	Uint32 *dst;
	Uint32 code;

	if (0 <= num && num <= MAX_PRESET) {
		dst = vkkey_preset_map[tab][num];
	} else {
		return;
	}

	for(int row=0; row<rows; row++) {
		if (table[tab][row].enabled) {
			for(int col=0; col<cols; col++) {
				code = table[tab][row].cols[col].vk_keycode;
				dst[table[tab][row].vm_keycode * cols + col] = code;
			}
		}
	}
	if (tab == 1) {
		dst[(vkkey_defmap_rows[tab] - 1) * cols] = combi1;
	}

}

void AG_KEYBIND_DLG::click_ok(AG_Window *win)
{
	Uint32 code;

	AG_TableSetPollInterval(tbls[1], 0);

	for(int tab=0; tab<MAX_TAB_NUM; tab++) {
		int rows = vmkey_map_size[tab];
		int cols = vkkey_defmap_cols[tab];
		Uint32 *dst = vkkey_map[tab];

		for(int row=0; row<rows; row++) {
			if (table[tab][row].enabled) {
				for(int col=0; col<cols; col++) {
					code = table[tab][row].cols[col].vk_keycode;
					dst[table[tab][row].vm_keycode * cols + col] = code;
				}
			}
		}
		if (tab == 1) {
			dst[(vkkey_defmap_rows[tab] - 1) * cols] = combi1;
		}
	}

	emu->save_keybind();

	AG_ObjectDetach(win);
	gui->Pause(false);
}

void AG_KEYBIND_DLG::click_cancel(AG_Window *win)
{
	AG_TableSetPollInterval(tbls[1], 0);
	AG_ObjectDetach(win);
	gui->Pause(false);
}

void AG_KEYBIND_DLG::update_key(AG_Table *tbl, int sym, int mod, Uint32 unicode)
{
	int tab = selected_tab;
	int row = selected[tab].row;
	int col = selected[tab].col;
	int idx = row2idx_map[tab][row];
	int new_key = 0;
	codecols_t *obj;

	if (tab == 0 && col > 0) {
		// SDL̃Cxgf[^ɂ
		emu->translate_sdl_keysym(&gui->sdl_event,NULL,&new_key);
		obj = &table[tab][idx].cols[col-1];
		SetVkKeyCode(obj, new_key, NULL);
		Update();
	}
}

void AG_KEYBIND_DLG::update_joy()
{
	int tab = selected_tab;
	int row = selected[tab].row;
	int col = selected[tab].col;
	int idx = row2idx_map[tab][row];
	codecols_t *obj;

	if (tab == 1 && col > 0) {
		emu->reset_joystick();
//		emu->update_joystick();
		obj = &table[tab][idx].cols[col-1];
		if (joy_stat[col-1]) {
			SetVkJoyCode(obj, joy_stat[col-1], NULL);
			Update();
		}
	}
}

/*
 * Event Handler (static)
 */
void AG_KEYBIND_DLG::OnOk(AG_Event *event)
{
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->click_ok(win);
}

void AG_KEYBIND_DLG::OnClose(AG_Event *event)
{
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->click_cancel(win);
}

void AG_KEYBIND_DLG::OnUpdate(AG_Event *event)
{
//	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);

	dlg->update_joy();
}

void AG_KEYBIND_DLG::OnClickTab(AG_Event *event)
{
	AG_Notebook *tab = (AG_Notebook *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);

	for(int i=0; i<MAX_TAB_NUM; i++) {
		if (dlg->tabs[i] == tab->sel_tab) {
			dlg->selected_tab = i;
			break;
		}
	}
}

void AG_KEYBIND_DLG::OnClickCell(AG_Event *event)
{
	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	int row = AG_INT(2);
//	int col = AG_INT(3);	// oops...cannot get... bug?
	int col = 0;

	for(int i=0; i<3; i++) {
		if (AG_TableCellSelected(tbl, row, i)) {
			col = i;
			break;
		}
	}

	dlg->click_cell(tbl, row, col);
}

void AG_KEYBIND_DLG::OnKeyDown(AG_Event *event)
{
	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	int sym = AG_INT(2);
	int mod = AG_INT(3);
	Uint32 unicode = AG_UINT(4);

	dlg->update_key(tbl, sym, mod, (Uint32)unicode);
}

void AG_KEYBIND_DLG::OnLoadDefault(AG_Event *event)
{
//	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);

	dlg->load_data(-1);
}

void AG_KEYBIND_DLG::OnLoadPreset(AG_Event *event)
{
//	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	int num = AG_INT(2);

	dlg->load_data(num);
}

void AG_KEYBIND_DLG::OnSavePreset(AG_Event *event)
{
//	AG_Table *tbl = (AG_Table *)AG_SELF();
	AG_KEYBIND_DLG *dlg = (AG_KEYBIND_DLG *)AG_PTR(1);
	int num = AG_INT(2);

	dlg->save_data(num);
}
