/** @file ag_keybind_dlg.h

	Skelton for retropc emulator
	SDL + Agar edition

	@author Sasaji
	@date   2012.4.8

	@brief [ ag_keybind_dlg ]
*/

#ifndef _AG_KEYBIND_DLG_H_
#define _AG_KEYBIND_DLG_H_

#include "ag_dlg.h"

#define KBCTRL_MAX_LINES 128
#define KBCTRL_MAX_COLS  2

#define MAX_TAB_NUM 2
#define MAX_PRESET	4

typedef struct codecols_st {
	int     col;
	Uint32  vk_keycode;
	Uint32  vk_prev_keycode;
} codecols_t;

typedef struct codetable_st {
	int        row;
	codecols_t cols[KBCTRL_MAX_COLS];
	Uint16     vm_keycode;
	bool       enabled;
} codetable_t;

/**
	@brief keybind dialog
*/
class AG_KEYBIND_DLG : public AG_DLG {
	friend class AG_GUI_BASE;

private:
	codetable_t table[MAX_TAB_NUM][KBCTRL_MAX_LINES];
	int    vmkey_map_size[MAX_TAB_NUM];
	int		row2idx_map[MAX_TAB_NUM][KBCTRL_MAX_LINES];

	Uint32 *vkkey_defmap[MAX_TAB_NUM];
	int     vkkey_defmap_rows[MAX_TAB_NUM];
	int     vkkey_defmap_cols[MAX_TAB_NUM];

	Uint32 *vkkey_map[MAX_TAB_NUM];
	Uint32 *vkkey_preset_map[MAX_TAB_NUM][MAX_PRESET];

	int     combi1;

	AG_NotebookTab *tabs[MAX_TAB_NUM];
	int selected_tab;
	struct selected_st {
		int row;
		int col;
		AG_TableCell *cell;
	} selected[MAX_TAB_NUM];

	AG_Table *tbls[MAX_TAB_NUM];

	Uint32 tm1;

	Uint32 *joy_stat;	// joystick #1, #2 (b0 = up, b1 = down, b2 = left, b3 = right, b4-b31 = trigger #1-#28

	bool get_vmkeylabel(int, char *);
	bool get_vkkeylabel(Uint32, char *);
	bool get_vkjoylabel(Uint32, char *);
	Uint32 translate_vkkey(Uint32);

	void click_ok(AG_Window *);
	void click_cancel(AG_Window *);
	void click_cell(AG_Table *, int, int);

	void load_data(int);
	void save_data(int);

	void update_key(AG_Table *, int, int, Uint32);
	void update_joy();

	static void OnOk(AG_Event *);
	static void OnClose(AG_Event *);
	static void OnUpdate(AG_Event *);
	static void OnClickTab(AG_Event *);
	static void OnClickCell(AG_Event *);
	static void OnKeyDown(AG_Event *);
	static void OnLoadDefault(AG_Event *);
	static void OnLoadPreset(AG_Event *);
	static void OnSavePreset(AG_Event *);

public:
	AG_KEYBIND_DLG(EMU *, AG_GUI_BASE *);
	~AG_KEYBIND_DLG();

	void Create();
	void Update();

	void SetVmKeyMap(int tab_num, Uint16 *vmKeyMap, int size);
	void SetVmKey(int tab_num, int idx, Uint16 code);
	bool SetVmKeyCode(int tab_num, int idx, Uint16 code);

	void SetVkKeyMap(int tab_num, Uint32 *vkKeyMap);
	void SetVkKeyDefMap(int tab_num, Uint32 *vkKeyDefMap, int rows, int cols);
	void SetVkKeyPresetMap(int tab_num, Uint32 *vkKeyMap, int idx);
	bool SetVkKeyCode(codecols_t *obj, Uint32 code, char *label);
	bool SetVkJoyCode(codecols_t *obj, Uint32 code, char *label);
	void SetTitleLabel(const _TCHAR *vm, const _TCHAR *vk, const _TCHAR *vj);
};

#endif /* _AG_KEYBIND_DLG_H_ */
