/** @file ledbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.08 -

	@brief [ led box ]
*/
#include "ledbox.h"

#define RECT_IN(st, xx, yy, ww, hh) { \
	st.x = xx; \
	st.y = yy; \
	st.w = ww; \
	st.h = hh; \
}

LedBox::LedBox()
{
	surface = NULL;

	suf_base = NULL;

	suf_kata = NULL;
	suf_hira = NULL;
	suf_caps = NULL;

	suf_play = NULL;
	suf_rec  = NULL;

#ifdef USE_FD1
	suf_fddr = NULL;
	suf_fddg = NULL;
#endif

	flag_old = -1;
}

LedBox::~LedBox()
{
#ifdef USE_FD1
	if (suf_fddr != NULL) SDL_FreeSurface(suf_fddr);
	if (suf_fddg != NULL) SDL_FreeSurface(suf_fddg);
#endif
	if (suf_play != NULL) SDL_FreeSurface(suf_play);
	if (suf_rec  != NULL) SDL_FreeSurface(suf_rec);

	if (suf_kata != NULL) SDL_FreeSurface(suf_kata);
	if (suf_hira != NULL) SDL_FreeSurface(suf_hira);
	if (suf_caps != NULL) SDL_FreeSurface(suf_caps);

	if (suf_base != NULL) SDL_FreeSurface(suf_base);

	if (surface  != NULL) SDL_FreeSurface(surface);
}

bool LedBox::Create(char *app_path, SDL_PixelFormat *format)
{
	char path[_MAX_PATH];

	sprintf(path,"%s%s%s",app_path,RESDIR,"ledbox_4.bmp");
	suf_base = SDL_LoadBMP(path);
	if (suf_base == NULL) return false;

	sprintf(path,"%s%s%s",app_path,RESDIR,"led_r_h_red.bmp");
	suf_kata = SDL_LoadBMP(path);
	if (suf_kata == NULL) return false;
	RECT_IN(pt_kata, 3, 3, suf_kata->w, suf_kata->h);
	sprintf(path,"%s%s%s",app_path,RESDIR,"led_r_h_green.bmp");
	suf_hira = SDL_LoadBMP(path);
	if (suf_hira == NULL) return false;
	RECT_IN(pt_hira, 3, 11, suf_hira->w, suf_hira->h);
	sprintf(path,"%s%s%s",app_path,RESDIR,"led_rv_h_red.bmp");
	suf_caps = SDL_LoadBMP(path);
	if (suf_caps == NULL) return false;
	RECT_IN(pt_caps, 36, 3, suf_caps->w, suf_caps->h);

	sprintf(path,"%s%s%s",app_path,RESDIR,"led_rr_h_green.bmp");
	suf_play = SDL_LoadBMP(path);
	if (suf_play == NULL) return false;
	RECT_IN(pt_play, 65, 4, suf_play->w, suf_play->h);
	sprintf(path,"%s%s%s",app_path,RESDIR,"led_c_h_red.bmp");
	suf_rec  = SDL_LoadBMP(path);
	if (suf_rec == NULL) return false;
	RECT_IN(pt_rec,  98, 4, suf_rec->w, suf_rec->h);

#ifdef USE_FD1
	sprintf(path,"%s%s%s",app_path,RESDIR,"led_rv_h_red.bmp");
	suf_fddr = SDL_LoadBMP(path);
	if (suf_fddr == NULL) return false;
	sprintf(path,"%s%s%s",app_path,RESDIR,"led_rv_h_green.bmp");
	suf_fddg = SDL_LoadBMP(path);
	if (suf_fddg == NULL) return false;
	RECT_IN(pt_fdd1, 130, 3, suf_fddr->w, suf_fddr->h);
	RECT_IN(pt_fdd2, 155, 3, suf_fddr->w, suf_fddr->h);
#endif

	if (!CreateSurface(format)) {
		return false;
	}

	return true;
}

bool LedBox::CreateSurface(SDL_PixelFormat *format)
{
	if (suf_base == NULL) return false;
	if (surface != NULL) {
		SDL_FreeSurface(surface);
	}
	surface = SDL_ConvertSurface(suf_base, format, SDL_SWSURFACE);
	if (surface == NULL) return false;

	return true;
}

void LedBox::MixScreen(SDL_Surface *screen, int x, int y)
{
	SDL_Rect re = { x, y, surface->w, surface->h };

	SDL_BlitSurface(surface, NULL, screen, &re);
}

void LedBox::Update(SDL_Surface *screen, int flag, int x, int y)
{
	if (flag != flag_old) {
		SDL_BlitSurface(suf_base, NULL, surface, NULL);

		// katakana
		if (flag & 0x0001) {
			SDL_BlitSurface(suf_kata, NULL, surface, &pt_kata);
		}
		// hiragana
		if (flag & 0x0002) {
			SDL_BlitSurface(suf_hira, NULL, surface, &pt_hira);
		}
		// caps
		if ((flag & 0x0004) == 0) {
			SDL_BlitSurface(suf_caps, NULL, surface, &pt_caps);
		}
		// play
		if (flag & 0x0010) {
			SDL_BlitSurface(suf_play, NULL, surface, &pt_play);
		}
		// rec
		if (flag & 0x0020) {
			SDL_BlitSurface(suf_rec , NULL, surface, &pt_rec);
		}
#ifdef USE_FD1
		// fdd1
		if ((flag & 0x1100) == 0x0100) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd1);
		} else if ((flag & 0x1100) == 0x1100) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd1);
		}
#endif
#ifdef USE_FD2
		// fdd2
		if ((flag & 0x2200) == 0x0200) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd2);
		} else if ((flag & 0x2200) == 0x2200) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd2);
		}
#endif
	}
	flag_old = flag;

	MixScreen(screen, x, y);
}
