/** @file msgboard.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.06.26 -

	@brief [ message board ]
*/
#include "msgboard.h"
#include "emu.h"
#include "config.h"

#define RECT_IN(st, xx, yy, ww, hh) { \
	st.x = xx; \
	st.y = yy; \
	st.w = ww; \
	st.h = hh; \
}

#define RGB_IN(st, rr, gg, bb) { \
	st.r = rr; \
	st.g = gg; \
	st.b = bb; \
}

MsgBoard::MsgBoard(EMU *pEmu)
{
	visible = true;
	enable  = false;

	sMainSuf = NULL;
	emu = pEmu;

	msg.font = NULL;
	info.font = NULL;

	inited = false;
}

MsgBoard::~MsgBoard()
{
	if (inited) {
		if (msg.font != NULL) TTF_CloseFont(msg.font);
		if (info.font != NULL) TTF_CloseFont(info.font);
		TTF_Quit();
	}
	if (sMainSuf != NULL) SDL_FreeSurface(sMainSuf);
}

void MsgBoard::CreateSurface(SDL_PixelFormat *format, int width, int height)
{
	if (sMainSuf != NULL) {
		SDL_FreeSurface(sMainSuf);
	}
	sMainSuf = SDL_CreateRGBSurface(SDL_SWSURFACE, width, 64, format->BitsPerPixel,
			format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (sMainSuf == NULL) {
		enable = false;
	} else {
		enable = true;
	}
}

// ʏ
void MsgBoard::InitScreen(SDL_PixelFormat *format, int width, int height)
{
	// F
	RGB_IN(fg, 0x00, 0xc0, 0x80);
	RGB_IN(bg, 0x00, 0x40, 0x00);

	// EBhETCY
	szWin.cx = width;
	szWin.cy = height;

	CreateSurface(format, width, height);

	if (enable) {
		// SDL_ttf 
		if (!TTF_WasInit()) {
			if (TTF_Init() != -1) {
				inited = true;
			} else {
				enable = false;
			}
		}
	}
	if (enable) {
		// bZ[WptHg̐ݒ
		enable = set_sys_font(12, &msg.font);
		if (enable) {
			TTF_SetFontStyle(msg.font, TTF_STYLE_NORMAL);
		}
		// gݒ
		RECT_IN(msg.re, 0, 0, width, 31);
		// \ʒu
		msg.pt.x = 0; msg.pt.y = 0; info.place = 2;
	}
	if (enable) {
		// ptHg̐ݒ
		enable = set_sys_font(22, &info.font);
		if (enable) {
			TTF_SetFontStyle(info.font, TTF_STYLE_NORMAL);
		}
		// gݒ
		RECT_IN(info.re, 0, 32, width, 63);
		// \ʒu
		info.pt.x = 0; info.pt.y = 0; info.place = 1;
	}
	if (enable) {
		emu->out_log(LOG_INFO , _T("MsgBoard: OK"));
	} else {
		emu->out_log(LOG_ERROR, _T("MsgBoard: Failed"));
	}
}

// o
void MsgBoard::draw(SDL_Surface *screen, msg_data_t &data)
{
	SDL_Rect reDst;

	if (!data.lists.empty()) {
//		list_t::iterator it = data.lists.begin();

		// ʒǔvZ
		if (data.place & 1) {
			reDst.x = szWin.cx + data.pt.x - data.sz.cx;
			reDst.w = data.sz.cx;
		} else {
			reDst.x = data.pt.x;
			reDst.w = data.sz.cx;
		}
		if (data.place & 2) {
			reDst.y = szWin.cy + data.pt.y - data.sz.cy;
			reDst.h = data.sz.cy;
		} else {
			reDst.y = data.pt.y;
			reDst.h = data.sz.cy;
		}

		// CReLXgɃbZ[WRs[
		SDL_BlitSurface(sMainSuf, &data.re,
			screen, &reDst);
	}
}

void MsgBoard::Draw(SDL_Surface *screen)
{
	if (!enable || !visible) return;

	draw(screen, msg);
	draw(screen, info);
}

// obNobt@ɕ`
void MsgBoard::draw_text(msg_data_t &data)
{
	SDL_Surface *sTextSuf = NULL;

	int cx = 0;
	int cy = 0;
	int len = 0;

	if (enable && !data.lists.empty()) {
		list_t::iterator it = data.lists.begin();

		// g`
		SDL_FillRect(sMainSuf, &data.re, SDL_MapRGB(sMainSuf->format, bg.r, bg.g, bg.b));

		// `
		len = (int)_tcslen(it->msg);
		sTextSuf = TTF_RenderUTF8_Shaded(data.font, it->msg, fg, bg);
		if (sTextSuf != NULL) {
			SDL_BlitSurface(sTextSuf, NULL, sMainSuf, &data.re);

			// ̘̕gvZ
			TTF_SizeUTF8(data.font, it->msg, &cx, &cy);
			data.sz.cx = cx + 4;
			data.sz.cy = cy + 4;

			SDL_FreeSurface(sTextSuf);

		} else {
			// G[
			data.sz.cx = 0;
			data.sz.cy = 0;
		}
	}
}

// bZ[Wݒ
void MsgBoard::Set(msg_data_t &data, const _TCHAR *str, int sec)
{
	item_t itm;

	_tcsncpy(itm.msg, str, 1023);
	itm.cnt = (60 * sec);

	data.lists.push_front(itm);
	if (data.lists.size() > 10) {
		data.lists.pop_back();
	}

	if (enable) {
		draw_text(data);
	}
}
void MsgBoard::SetMessage(const _TCHAR *str, int sec)
{
	Set(msg, str, sec);
}
void MsgBoard::SetInfo(const _TCHAR *str, int sec)
{
	Set(info, str, sec);
}
void MsgBoard::SetMessageF(const _TCHAR *format, ...)
{
	_TCHAR buf[1024];

	va_list ap;

	va_start(ap, format);
	_vstprintf(buf, format, ap);
	va_end(ap);

	SetMessage(buf);
}
void MsgBoard::SetInfoF(const _TCHAR *format, ...)
{
	_TCHAR buf[1024];

	va_list ap;

	va_start(ap, format);
	_vstprintf(buf, format, ap);
	va_end(ap);

	SetInfo(buf);
}

// EBhETCYݒ
void MsgBoard::SetSize(int width, int height)
{
	szWin.cx = width;
	szWin.cy = height;
}

// bZ[W`ʒuݒ
void MsgBoard::SetMessagePos(int cx, int cy, int place)
{
	msg.pt.x = cx;
	msg.pt.y = cy;
	msg.place = place;
}
void MsgBoard::SetInfoPos(int cx, int cy, int place)
{
	info.pt.x = cx;
	info.pt.y = cy;
	info.place = place;
}

// bZ[W폜
void MsgBoard::Delete(msg_data_t &data, const _TCHAR *str)
{
	bool redraw = false;
	if (!data.lists.empty()) {
		list_t::iterator it = data.lists.begin();
		list_t::iterator it_next;
		while(it != data.lists.end())	{
			it_next = it;
			it_next++;

			if (_tcscmp(str, it->msg) == 0) {
				data.lists.erase(it);
				redraw = true;
			}
			it = it_next;
		}
	}
	if (redraw) {
		draw_text(data);
	}
}
void MsgBoard::DeleteMessage(const _TCHAR *str)
{
	Delete(msg, str);
}
void MsgBoard::DeleteInfo(const _TCHAR *str)
{
	Delete(info, str);
}

// JEg_E
void MsgBoard::count_down(msg_data_t &data)
{
	bool redraw = false;
	if (!data.lists.empty()) {
		list_t::iterator it = data.lists.begin();
		list_t::iterator it_next;
		while(it != data.lists.end())	{
			it_next = it;
			it_next++;

			if (it->cnt > 0) it->cnt--;
			if (it->cnt == 0) {
				data.lists.erase(it);
				redraw = true;
			}
			it = it_next;
		}
	}
	if (redraw) {
		draw_text(data);
	}
}
void MsgBoard::CountDown(void)
{
	count_down(msg);
	count_down(info);
}

// ʕ\pVXetHg̐ݒ
bool MsgBoard::set_sys_font(int pt, TTF_Font **font)
{
	sbuf_t sbuf;
	std::list<sbuf_t> fpath;
	std::list<sbuf_t> fname;
	TTF_Font *f;

#if defined(_WIN32)
	sprintf(sbuf.buf, "%s%s", emu->application_path(), RESDIR);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s", emu->application_path());
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s\\fonts\\", SDL_getenv("SystemRoot"));
	fpath.push_back(sbuf);

	strcpy(sbuf.buf, config.msgboard_fontname);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "arial.ttf");
	fname.push_back(sbuf);

#elif defined(linux)
	sprintf(sbuf.buf, "%s%s", emu->application_path(), RESDIR);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s", emu->application_path());
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/.fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/truetype/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/X11R6/lib/X11/fonts/TTF/");
	fpath.push_back(sbuf);

	strcpy(sbuf.buf, config.msgboard_fontname);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "ttf-japanese-gothic.ttf");
	fname.push_back(sbuf);

#elif defined(__APPLE__) && defined(__MACH__)
	sprintf(sbuf.buf, "%s%s", emu->application_path(), RESDIR);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s", emu->application_path());
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/Library/Fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/Library/Fonts/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/System/Library/Fonts/");
	fpath.push_back(sbuf);

	strcpy(sbuf.buf, config.msgboard_fontname);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "AppleGothic.ttf");
	fname.push_back(sbuf);

#elif defined(__FreeBSD__)
	sprintf(sbuf.buf, "%s%s", emu->application_path(), RESDIR);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s", emu->application_path());
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/.fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/TTF/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/local/lib/X11/fonts/TTF/");
	fpath.push_back(sbuf);

	strcpy(sbuf.buf, config.msgboard_fontname);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "luxisr.ttf");
	fname.push_back(sbuf);

#endif
	std::list<sbuf_t>::iterator itn = fname.begin();
	while (itn != fname.end()) {
		std::list<sbuf_t>::iterator itp = fpath.begin();
		while (itp != fpath.end()) {
			sprintf(sbuf.buf, "%s%s", itp->buf, itn->buf);
			f = TTF_OpenFont(sbuf.buf, pt);
			if (f != NULL) {
				*font = f;
				emu->out_log(LOG_INFO, _T("MsgBoard: Use %s"), sbuf.buf);
				return true;
			}
			itp++;
		}
		itn++;
	}
	return false;
}
