/** @file cmt.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.13 -

	@brief [ cmt ]
*/

#include "cmt.h"

void CMT::initialize()
{
	// data recorder
	fio = new FILEIO();
	wav = new PARSEWAV();
	memset(data_buffer, 0x00, sizeof(data_buffer));
	memset(c_onedata, 0x00, sizeof(c_onedata));
	data_read_pos = 0;
	data_write_pos = 0;
	data_read_size = 0;
	data_write_size = 0;
	is_wav = is_l3c = is_l3b = false;
	data_len = 0;

	buffer_overflow = false;
	play = rec = false;
	remote = false;
	remote_prev = false;
	send_ok = true;
	recv_ok = false;

	baud_sel = 0;

	register_id = -1;
//	register_event_by_clock(this, 0, (int)(CPU_CLOCKS / 19200), true, &register_id);

	relay_data[0] = NULL;
	relay_size[0] = 0;
	relay_data[1] = NULL;
	relay_size[1] = 0;
	relay_play_pos = 0;
	relay_play = 0;

	tape_wave_buf = NULL;
	tape_wave_buf_wpos = 0;
	tape_wave_buf_rpos = 0;
}

void CMT::reset()
{
	baud_sel = 0;
	if (register_id != -1) {
		cancel_my_event();
		register_my_event();
	}
	remote = true;
}

void CMT::release()
{
	close_datarec();
	cancel_my_event();

	delete[] tape_wave_buf;

	delete[] relay_data[1];
	delete[] relay_data[0];

	delete wav;
	delete fio;
}

void CMT::load_relay_wav()
{
	// load relay wav file
	_TCHAR *app_path, *rom_path[2], file_path[_MAX_PATH];
	wav_header_t	 head;
	wav_fmt_chank_t  fmt;
	wav_data_chank_t data;
	size_t			 data_len;

	relay_wav_enable = 0;

	relay_data[0] = new uint8[sample_rate >> 2];	// 0.25sec
	relay_data[1] = new uint8[sample_rate >> 2];	// 0.25sec

	rom_path[0] = config.rom_path;
	rom_path[1] = emu->application_path();

	FILEIO* fre = new FILEIO();
	for(int i=0; i<2; i++) {
		app_path = rom_path[i];

		if ((relay_wav_enable & 1) == 0) {
			_stprintf(file_path, _T("%srelay_on.wav"), app_path);
			if(fre->Fopen(file_path, FILEIO_READ_BINARY)) {
				if (wav->CheckWavFormat(fre, &head, &fmt, &data, &data_len) == 0) {
					relay_size[0] = read_relay_wav_data(fre, &fmt, data_len, relay_data[0]);
					relay_wav_enable |= 1;
					emu->out_log(LOG_INFO,_T("relay_on.wav was loaded."));
				}
				fre->Fclose();
			}
		}
		if ((relay_wav_enable & 2) == 0) {
			_stprintf(file_path, _T("%srelayoff.wav"), app_path);
			if(fre->Fopen(file_path, FILEIO_READ_BINARY)) {
				if (wav->CheckWavFormat(fre, &head, &fmt, &data, &data_len) == 0) {
					relay_size[1] = read_relay_wav_data(fre, &fmt, data_len, relay_data[1]);
					relay_wav_enable |= 2;
					emu->out_log(LOG_INFO,_T("relayoff.wav was loaded."));
				}
				fre->Fclose();
			}
		}
	}
	delete fre;

#ifdef _DEBUG
	if ((relay_wav_enable & 1) == 0) {
		emu->out_debug(_T("relay_on.wav couldn't be loaded."));
	}
	if ((relay_wav_enable & 2) == 0) {
		emu->out_debug(_T("relayoff.wav couldn't be loaded."));
	}
#endif
}

int CMT::read_relay_wav_data(FILEIO *fre, wav_fmt_chank_t *fmt, size_t data_len, uint8 *outdata)
{
	uint8 buf[32];
	int outpos = 0;
	size_t len = 0;
	size_t siz = 0;

	wav->InitConvSampleData();
	while ((siz = fre->Fread(buf, sizeof(uint8), 32)) > 0 && len < data_len && outpos < (sample_rate >> 2)) {
		outpos = wav->OutConvSampleData(buf, fmt->sample_rate, fmt->sample_bits, fmt->channels, siz * 8 / fmt->sample_bits
									, outdata, sample_rate, 8, (sample_rate >> 2), outpos);
		len += siz;
	}
	return outpos;
}

void CMT::write_io8(uint32 addr, uint32 data)
{
	set_data(data);
}

void CMT::set_data(uint32 data)
{
	int len;

	if (is_l3b) {
		data |= 0x30;
	}
	if (remote && recv_ok && rec) {
		if (config.realmode_datarec) {
			set_tape_wave(data);
		}
		if (data_write_pos >= CMTDATA_BUFFER_SIZE) {
			buffer_overflow = true;
			if (d_ctrl != NULL) {
				d_ctrl->write_signal(SIG_ACIA_ERROR, 1, 1);
			}
		} else {
			if (is_l3c || is_wav) {
				if (is_l3c && data_len >= 88) {
					// cr + lf
					data_buffer[data_write_pos]='\r';
					data_write_pos++;
					data_buffer[data_write_pos]='\n';
					data_write_pos++;
					data_len = 0;
				}
				wav->param.baud = baud_sel;
				len = wav->SetData(data, &data_buffer[data_write_pos]);
				data_write_pos+=len;
				data_len+=len;
			} else {
				if (is_l3b && data_len >= 110
				&& (data_buffer[data_write_pos - 1] & 1) == 1
				&& (data & 1) == 0) {
					// cr + lf
					data_buffer[data_write_pos]='\r';
					data_write_pos++;
					data_buffer[data_write_pos]='\n';
					data_write_pos++;
					data_len = 0;
				}
				data_buffer[data_write_pos]=data & 0xff;
				data_write_pos++;
				data_len++;
			}
		}
	}
}

uint32 CMT::read_io8(uint32 addr)
{
	return get_data();
}

uint32 CMT::get_data()
{
	uint32 data = 0;
	static int cnt = 0;

	// read 1byte from buffer
	if (remote) {
		while (data_read_pos < data_read_size) {
			data = data_buffer[data_read_pos];
			data_read_pos++;

			// read from file
			if (data_read_pos == data_read_size) {
//				remain -= data_read_size;
				load_image();
			}

//emu->out_debug("%02x",data);
			if (is_l3c || is_wav) {
				if (config.realmode_datarec) {
					set_tape_amp(c_onedata);
				}
				break;
			}

			if (!is_l3b) {
				if (config.realmode_datarec) {
					set_tape_wave(data);
				}
				break;
			}
			// parse l3b
			if ((data & 0xff) != 0x0a && (data & 0xff) != 0x0d) {
				if (config.realmode_datarec) {
					set_tape_wave(data);
				}
				break;
			}
		}
	} else {
		data = cnt ? 0xff : 0;
		cnt--;
		if (cnt < 0) cnt = 15;
	}
	return data;
}

void CMT::write_signal(int id, uint32 data, uint32 mask)
{
	// remote on/off
	switch(id) {
		case SIG_CMT_REMOTE:
			remote_prev = remote;
			remote = (data & mask) ? true : false;
			break;
		case SIG_CMT_BAUD_SEL:
			baud_sel = (data & mask) ? 1 : 0;
			if (register_id != -1) {
				cancel_my_event();
				register_my_event();
			}
			break;
		case SIG_ACIA_DTR:
			send_ok = (data & mask) ? true : false;
			break;
		case SIG_ACIA_RTS:
			recv_ok = (data & mask) ? true : false;
			break;
		case SIG_ACIA_TXDATA:
			// receive from acia
			set_data(data);
			break;
		case SIG_CPU_RESET:
			now_reset = (data & mask) ? true : false;
			if (now_reset) {
				reset();
			}
			break;
	}

}

bool CMT::play_datarec(_TCHAR* filename)
{
	close_datarec();

	if(fio->Fopen(filename, FILEIO_READ_BINARY)) {
		is_wav = is_l3c = is_l3b = false;
		is_wav = check_extension(filename, _T(".wav"));
		is_l3c = check_extension(filename, _T(".l3c"));
		is_l3b = check_extension(filename, _T(".l3b"));

		// load image file
		play = true;
		register_my_event();

//		fio->Fseek(0, FILEIO_SEEK_END);
//		remain = fio->Ftell();
//		fio->Fseek(0, FILEIO_SEEK_SET);
		play = set_load_param();
		if (play) load_image();
	}
	return play;
}

void CMT::rewind_datarec()
{
	if (play) {
		fio->Fseek(0, FILEIO_SEEK_SET);

		play = set_load_param();
		load_image();
	}
	if (rec) {
		fio->Fseek(0, FILEIO_SEEK_SET);

		data_write_pos = 0;
		data_len = 0;

		set_save_param();
	}
}

void CMT::fast_forward_datarec()
{
	if (play) {
		fio->Fseek(0, FILEIO_SEEK_END);

		load_image();
	}
	if (rec) {
		fio->Fseek(0, FILEIO_SEEK_END);

		data_write_pos = 0;
	}
}

void CMT::realmode_datarec()
{
	config.realmode_datarec = !config.realmode_datarec;
	if (register_id != -1) {
		cancel_my_event();
		register_my_event();
	}
}

bool CMT::rec_datarec(_TCHAR* filename)
{
	close_datarec();

	if(fio->Fopen(filename, FILEIO_READ_WRITE_BINARY)
	|| fio->Fopen(filename, FILEIO_WRITE_BINARY)) {
		is_wav = is_l3c = is_l3b = false;
		is_wav = check_extension(filename, _T(".wav"));
		is_l3c = check_extension(filename, _T(".l3c"));
		is_l3b = check_extension(filename, _T(".l3b"));

		data_write_pos = 0;
		data_len = 0;
		rec = true;

		register_my_event();

		set_save_param();
	}

	return rec;
}

void CMT::close_datarec()
{
	// close file
	if(rec) {
		save_image();
		if (is_wav) {
			wav->SetFileHeader(fio);
		}
	}

	if(play || rec) {
		cancel_my_event();
		fio->Fclose();
	}
	play = rec = false;
}

bool CMT::set_load_param()
{
	if (is_l3c || is_wav) {
		if (wav->CheckFileFormat(fio, (is_l3c ? 1 : 0)) != 0) {
			emu->out_log(LOG_ERROR, _T("%s"), wav->Errmsg());
			return false;
		}
		wav->param.reverse = emu->get_parami(EMU::ConfigBoxWavReverse);
		wav->param.half_wave = emu->get_parami(EMU::ConfigBoxWavHalf);
		wav->param.correct = emu->get_parami(EMU::ConfigBoxWavCorrect);
		wav->param.correct_type = emu->get_parami(EMU::ConfigBoxWavCorrectType);
		wav->InitData(fio, (is_l3c ? 1 : 0));
	}
	return true;
}

bool CMT::set_save_param()
{
	if (is_l3c || is_wav) {
		wav->param.sample_rate = emu->get_parami(EMU::ConfigBoxWavSampleRate);
		wav->param.sample_bits = emu->get_parami(EMU::ConfigBoxWavSampleBits);
		wav->InitData(fio, (is_l3c ? 1 : 0));
	}
	if (is_wav) {
		data_write_pos = wav->InitFileHeader(data_buffer);
		data_len = data_write_pos;
	}
	return true;
}

// binary image
void CMT::load_image()
{
	memset(data_buffer, 0x00, sizeof(data_buffer));
	if (is_l3c || is_wav) {
		wav->param.baud = baud_sel;
		c_onedata[0]='\0';
		data_buffer[0] = wav->GetData(c_onedata);
		data_read_size = 1;
		data_read_pos = 0;
		data_write_pos = 0;
		data_len = 0;
	} else {
		data_read_size = (int)fio->Fread(data_buffer, sizeof(uint8), sizeof(data_buffer));
		data_read_pos = 0;
		data_write_pos = 0;
		data_len = 0;
	}
}

void CMT::save_image()
{
	// binary image
	if (data_write_pos > 0) {
		fio->Fwrite(data_buffer, data_write_pos, 1);
		data_write_pos = 0;
	}
}

bool CMT::check_extension(_TCHAR* filename, const _TCHAR* ext)
{
	int nam_len = (int)_tcslen(filename);
	int ext_len = (int)_tcslen(ext);

	if(nam_len >= ext_len) {
		if(_tcsncmp(&filename[nam_len - ext_len], ext, ext_len) == 0) {
			return true;
		}
	}
	return false;
}

// ----------------------------------------------------------------------------
void CMT::register_my_event()
{
	int period = 1;

	if(register_id == -1) {
		if (config.realmode_datarec) {
			if (is_l3b || is_l3c || is_wav) {
				period = (int)(CPU_CLOCKS / (600 << baud_sel));
			} else {
				period = (int)(CPU_CLOCKS / (600 << baud_sel) * 11);
			}
		} else {
			period = (int)(CPU_CLOCKS / 19200);
		}
		register_event_by_clock(this, 0, period, true, &register_id);
	}
}

void CMT::cancel_my_event()
{
	if(register_id != -1) {
		cancel_event(register_id);
	}
	register_id = -1;
}

//void CMT::update_event()
//{
//	if(remote && send_ok && play) {
//		if(register_id == -1) {
//			int period = (int)(CPU_CLOCKS / 9600);
//			register_event_by_clock(this, 0, period, true, &register_id);
//		}
//	}
//	else {
//		if(register_id != -1) {
//			cancel_event(register_id);
//		}
//		register_id = -1;
//	}
//
//	// end of tape ?
//	bool signal = (play && remain == 0);
//	write_signals(&outputs_end, signal ? 0xffffffff : 0);
//}

// ----------------------------------------------------------------------------
// event handler
// ----------------------------------------------------------------------------

void CMT::event_callback(int event_id, int err)
{
	uint32 data = 0;
	uint32 mask = 0;
	if(remote) {
		// send to acia
		if (d_ctrl != NULL) {
			// send to rxclk
			d_ctrl->write_signal(SIG_ACIA_RXCLK, 1, 1);

			if (play && (send_ok || now_reset)) {
				// read data
				data = get_data();

				if (is_l3b || is_l3c || is_wav) {
					mask = 1;
				} else {
					mask = 0xff;
				}
				// send data to acia
				d_ctrl->write_signal(SIG_ACIA_RXDATA, data, mask);
			}
		}
		// receive from acia
		// send to txclk then acia send data to me
		if (d_ctrl != NULL) {
			if (is_l3b || is_l3c || is_wav) {
				mask = 1;
			} else {
				mask = 0xff;
			}
			d_ctrl->write_signal(SIG_ACIA_TXCLK, 1, mask);
		}
		if (rec) {
			// write data to file
			if (data_write_pos > 11) {
				save_image();
			}
		}
	}
//	update_event();
}

// ----------------------------------------------------------------------------
void CMT::mix(int32* buffer, int cnt)
{
	int32 *buffer_orig = buffer;

	// relay wave sound
	if ((relay_wav_enable & 1) != 0 && remote == true && remote_prev == false) {
		relay_play = 1;
		relay_play_pos = 0;
	} else if ((relay_wav_enable & 2) != 0 && remote == false && remote_prev == true) {
		relay_play = 2;
		relay_play_pos = 0;
	}
	remote_prev = remote;

	if (relay_play) {
		for(int i=0; i<cnt; i++) {
			*buffer++ += ((int)relay_data[relay_play-1][relay_play_pos] - 128) * max_vol * relay_volume / 12800;
			*buffer++ += ((int)relay_data[relay_play-1][relay_play_pos] - 128) * max_vol * relay_volume / 12800;
			relay_play_pos++;
			if (relay_play_pos >= relay_size[relay_play-1]) {
				break;
			}
		}
		if (relay_play_pos >= relay_size[relay_play-1]) {
			relay_play = 0;
		}
	}

	// tape wave sound
	if (tape_wave_buf_wpos > tape_wave_buf_rpos) {
		buffer = buffer_orig;
		for(int i=0; i<cnt; i++) {
			*buffer++ += tape_wave_buf[tape_wave_buf_rpos];
			*buffer++ += tape_wave_buf[tape_wave_buf_rpos];
			tape_wave_buf_rpos++;
			if (tape_wave_buf_wpos <= tape_wave_buf_rpos) {
				break;
			}
		}
		// shift
		if (tape_wave_buf_wpos > (TAPEWAV_BUFFER_SIZE/2)) {
			for(int pos=tape_wave_buf_rpos; pos<tape_wave_buf_wpos; pos++) {
				tape_wave_buf[pos-tape_wave_buf_rpos] = tape_wave_buf[pos];
			}
			tape_wave_buf_wpos -= tape_wave_buf_rpos;
			tape_wave_buf_rpos = 0;
		}
	} else {
		surplus = 0;
	}
}

void CMT::set_volume(int volr, int volt)
{
	relay_volume = volr;
	tape_volume = volt;
}

void CMT::init(int rate, int volume)
{
	sample_rate = rate;
	max_vol = volume;
	relay_volume = 0;
	tape_volume = 0;

	// load relay wav file
	load_relay_wav();

	// create tape wave
	create_tape_wave();
}

// ----------------------------------------------------------------------------
void CMT::create_tape_wave()
{
	tape_wave_buf = new int16[TAPEWAV_BUFFER_SIZE + 1];

	devide = 0;
	surplus = 0;
	freq_prev = 2400;
}

void CMT::set_tape_wave(uint32 data)
{
	if (is_l3b || is_l3c || is_wav) {
		// 1bit data
		if (data & 1) {
			// 2400Hz * 4
			set_tape_one_wave(4 >> baud_sel, 2400);
		} else {
			// 1200Hz * 2
			set_tape_one_wave(2 >> baud_sel, 1200);
		}
	} else {
		// 1byte data
		// startbit
		// 1200Hz * 2
		set_tape_one_wave(2 >> baud_sel, 1200);
		// databit
		for(int b=0; b<8; b++) {
			if (data & (1 << b)) {
				// 2400Hz * 4
				set_tape_one_wave(4 >> baud_sel, 2400);
			} else {
				// 1200Hz * 2
				set_tape_one_wave(2 >> baud_sel, 1200);
			}
		}
		// 2stopbit
		// 2400Hz * 4 * 2
		set_tape_one_wave(8 >> baud_sel, 2400);
	}
}

void CMT::set_tape_one_wave(int cnt, int freq)
{
	int a = max_vol * tape_volume / 100;

	// 1200Hz or 2400Hz
	if (freq > freq_prev) {
		surplus *= 2;
	} else if (freq < freq_prev) {
		surplus /= 2;
	}
	freq_prev = freq;

	freq *= 2;

	for(int j=0; j<cnt; j++) {
		devide = (sample_rate + surplus) / freq;
		surplus = (sample_rate + surplus) - (devide * freq);
		for(int i=0; i<devide; i++) {
			if (tape_wave_buf_wpos >= TAPEWAV_BUFFER_SIZE) {
				break;
			}
			tape_wave_buf[tape_wave_buf_wpos] = a;
			tape_wave_buf_wpos++;
		}
		devide = (sample_rate + surplus) / freq;
		surplus = (sample_rate + surplus) - (devide * freq);
		for(int i=0; i<devide; i++) {
			if (tape_wave_buf_wpos >= TAPEWAV_BUFFER_SIZE) {
				break;
			}
			tape_wave_buf[tape_wave_buf_wpos] = -a;
			tape_wave_buf_wpos++;
		}
	}
}

void CMT::set_tape_amp(char *c_data)
{
	char *p = c_data;

	while(p != NULL && *p != '\0') {
		set_tape_one_amp(*p);
		p++;
	}
}

void CMT::set_tape_one_amp(int data)
{
	int a = max_vol * tape_volume / 100;
	int freq = 4800;

	if (data & 1) {
		devide = (sample_rate + surplus) / freq;
		surplus = (sample_rate + surplus) - (devide * freq);
		for(int i=0; i<devide; i++) {
			if (tape_wave_buf_wpos >= TAPEWAV_BUFFER_SIZE) {
				break;
			}
			tape_wave_buf[tape_wave_buf_wpos] = a;
			tape_wave_buf_wpos++;
		}
	} else {
		devide = (sample_rate + surplus) / freq;
		surplus = (sample_rate + surplus) - (devide * freq);
		for(int i=0; i<devide; i++) {
			if (tape_wave_buf_wpos >= TAPEWAV_BUFFER_SIZE) {
				break;
			}
			tape_wave_buf[tape_wave_buf_wpos] = -a;
			tape_wave_buf_wpos++;
		}
	}
}

// ----------------------------------------------------------------------------
uint8 CMT::get_cmt_mode()
{
	return (remote && play ? 0x01 : 0) | (remote && rec ? 0x02 : 0);
}

// ----------------------------------------------------------------------------

void CMT::save_state(FILEIO *fp)
{
	struct vm_state_st vm_state;

	//
	vm_state_ident.version = 1;
	vm_state_ident.size = sizeof(vm_state_ident) + sizeof(vm_state);

	// copy values
	memset(&vm_state, 0, sizeof(vm_state));
	vm_state.flags = (remote ? 1 : 0) | (play ? 2 : 0) | (rec ? 4 : 0) | (baud_sel ? 8 : 0);
	vm_state.flags |= (send_ok ? 0x10 : 0) | (recv_ok ? 0x20 : 0);

	vm_state.register_id = register_id;

	fp->Fwrite(&vm_state_ident, sizeof(vm_state_ident), 1);
	fp->Fwrite(&vm_state, sizeof(vm_state), 1);
}

void CMT::load_state(FILEIO *fp)
{
	vm_state_ident_t vm_state_i;
	struct vm_state_st vm_state;

	READ_STATE_CHUNK(fp, vm_state_i, vm_state);

	// copy values
	remote = (vm_state.flags & 1) ? true : false;
//	play   = (vm_state.flags & 2) ? true : false;
//	rec    = false;	// ignore record mode
	baud_sel = (vm_state.flags & 8) ? 1 : 0;
	send_ok = (vm_state.flags & 0x10) ? true : false;
	recv_ok = (vm_state.flags & 0x20) ? true : false;

	register_id = vm_state.register_id;
}
