/** @file cmt.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.13 -

	@brief [ cmt ]
*/

#ifndef _CMT_H_
#define _CMT_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"
#include "acia.h"
#include "parsewav.h"

#define CMTDATA_BUFFER_SIZE	1024
#define WAVE_PARSE_SIZE		170
#define SIG_CMT_REMOTE		1
#define SIG_CMT_BAUD_SEL	2

#define TAPEWAV_BUFFER_SIZE	2048

class FILEIO;
class PARSEWAV;

/**
	@brief Cassette Magnetic Tape
*/
class CMT : public DEVICE
{
private:
	DEVICE* d_ctrl;

	uint8  data_buffer[CMTDATA_BUFFER_SIZE];
	char   c_onedata[CMTDATA_BUFFER_SIZE];
	int    data_read_pos;
	int    data_read_size;
	int    data_write_pos;
	int    data_write_size;
	bool   buffer_overflow;
	bool   is_wav;
	bool   is_l3c, is_l3b;
	int    data_len;

	FILEIO   *fio;
	PARSEWAV *wav;
	bool play, rec;
	bool remote;		// remote control
	bool remote_prev;
	uint8 baud_sel;		// baud select 0:600 1:1200
	bool send_ok;		// send to acia
	bool recv_ok;		// receive from acia

	int sample_rate;
	int max_vol;
	int relay_volume;
	int tape_volume;
	int relay_wav_enable;
	uint8 *relay_data[2];
	int    relay_size[2];
	int    relay_play_pos;
	int    relay_play;

	int16 *tape_wave_buf;
	int   tape_wave_buf_wpos;
	int   tape_wave_buf_rpos;

	int   devide;
	int   surplus;
	int   freq_prev;

	int register_id;

	//for resume
#pragma pack(1)
	struct vm_state_st {
		int register_id;
		uint8 flags;

		char  reserved[11];
	};
#pragma pack()
	bool  set_load_param();
	bool  set_save_param();
	void  load_image();
	void  save_image();
	void	set_data(uint32);
	uint32	get_data();
	bool check_extension(_TCHAR* filename, const _TCHAR* ext);

	void register_my_event();
	void cancel_my_event();

	void load_relay_wav();
	int  read_relay_wav_data(FILEIO *fre, wav_fmt_chank_t *fmt, size_t data_len, uint8 *outdata);
public:
	CMT(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("CMT");
		d_ctrl = NULL;
	}
	~CMT() {}

	// common functions
	void initialize();
	void reset();
	void release();

	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);

	// unique functions
	void set_context_ctrl(DEVICE* device) {
		d_ctrl = device;
	}
	bool play_datarec(_TCHAR* filename);
	bool rec_datarec(_TCHAR* filename);
	void close_datarec();
	void rewind_datarec();
	void fast_forward_datarec();
	void realmode_datarec();

	void event_callback(int , int);
	void mix(int32* buffer, int cnt);
	void set_volume(int volr, int volt);
	void init(int rate, int volume);

	void create_tape_wave();
	void set_tape_wave(uint32 data);
	void set_tape_one_wave(int cnt, int freq);
	void set_tape_amp(char *c_data);
	void set_tape_one_amp(int data);

	uint8 get_cmt_mode();

	void save_state(FILEIO *fp);
	void load_state(FILEIO *fp);
};

#endif
