/** @file comm.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.08.20 -

	@brief [ comm ]
*/

#include "comm.h"

void COMM::initialize()
{
	// RS-232C
	is_rs = (cfg_num ? 1 : 0);
//	receive_data = 1;
	received = false;

	register_id = -1;

	connect = 0;
	client_ch = -1;
	server_ch = -1;

	register_frame_event(this);
}

void COMM::reset()
{
	is_rs = (cfg_num ? 1 : 0);
//	receive_data = 1;
	received = false;

	memset(send_buff, 0, sizeof(send_buff));
	send_buff_w_pos = 0;
	send_buff_r_pos = 0;
	memset(recv_buff, 0, sizeof(recv_buff));
	recv_buff_w_pos = 0;
	recv_buff_r_pos = 0;

	cancel_my_event();
}

void COMM::release()
{
}

void COMM::write_io8(uint32 addr, uint32 data)
{
	if (!(addr & 1)) {
		// Write Control Register
		cr = data & 0xff;
		if (is_rs) {
//			connect_socket();
			cancel_my_event();
			register_my_event();
		}
	}
}

// relay
void COMM::write_signal(int id, uint32 data, uint32 mask)
{
	switch(id) {
		case SIG_COMM_RS:
			// from memory
			if (data & mask) {
				is_rs = 1;
				register_my_event();
			} else {
				is_rs = 0;
				cancel_my_event();
			}
			break;
		case SIG_ACIA_RXCLK:
		case SIG_ACIA_TXCLK:
		case SIG_ACIA_RXDATA:
		case SIG_ACIA_CTS:
		case SIG_ACIA_DCD:
			if (is_rs == 0) {
				// through cmt -> acia
				d_ctrl->write_signal(id, data, mask);
			}
			break;
		case SIG_ACIA_TXDATA:
			if (is_rs == 0) {
				// through acia -> cmt
				if (d_cmt) d_cmt->write_signal(id, data, mask);
			} else {
				send_data(data);
			}
			break;
		case SIG_ACIA_DTR:
		case SIG_ACIA_RTS:
			if (is_rs == 0) {
				// through acia -> cmt
				if (d_cmt) d_cmt->write_signal(id, data, mask);
			}
			break;
	}

}

void COMM::send_data(uint32 data)
{
	if (send_buff_w_pos < COMM_MAX_BUFF) {
		send_buff[send_buff_w_pos++] = (data & 0xff) | 0x30;
	}
//	emu->out_log(LOG_DEBUG, _T("comm send data=%d pos:%d"),data,send_buff_w_pos);
}

void COMM::recv_data(uint8 *data, size_t size)
{
#if 0
	if (is_rs) {
		for(size_t pos = 0; pos < size && recv_buff_w_pos < COMM_MAX_BUFF; pos++) {
			recv_buff[recv_buff_w_pos++] = data[pos];
		}
		received = true;

//		emu->out_debug("comm recv data size:%d",recv_buff_w_pos - recv_buff_r_pos);
	}
#endif
}

void COMM::register_my_event()
{
	if(register_id == -1) {
		int baud = (150 << config.comm_dipswitch[cfg_num]);
		switch(cr & 0x03) {
			case 0:
				baud *= 16;
				break;
			case 1:
				baud *= 4;
				break;
		}
		int period = (int)(CPU_CLOCKS / baud);
		register_event_by_clock(this, 0, period, true, &register_id);
//		emu->out_debug("set comm speed: %d baud",baud);
	}
}

void COMM::cancel_my_event()
{
	if(register_id != -1) {
		cancel_event(register_id);
	}
	register_id = -1;
}


// ----------------------------------------------------------------------------
// event handler
// ----------------------------------------------------------------------------

void COMM::event_frame()
{
	if (send_buff_r_pos < send_buff_w_pos) {
//		emu->out_log(LOG_DEBUG, _T("event frame send_buff ch:%d w:%d r:%d") ,client_ch,send_buff_w_pos,send_buff_r_pos);
		emu->send_data_tcp(client_ch);
//		emu->send_comm_data(&send_buff[send_buff_r_pos], send_buff_w_pos - send_buff_r_pos);
//		emu->out_debug("comm sended size:%d",send_buff_w_pos - send_buff_r_pos);
//		send_buff_w_pos = 0;
//		send_buff_r_pos = 0;
	}
	if (recv_buff_r_pos < recv_buff_w_pos) {
//		emu->out_log(LOG_DEBUG, _T("event frame recv_buff ch:%d w:%d r:%d") ,client_ch,recv_buff_w_pos,recv_buff_r_pos);
		memcpy(&recv_buff[0], &recv_buff[recv_buff_r_pos], recv_buff_w_pos - recv_buff_r_pos);
		recv_buff_w_pos -= recv_buff_r_pos;
		recv_buff_r_pos = 0;
	}
}

void COMM::event_callback(int event_id, int err)
{
	// send to acia
	if (d_ctrl != NULL) {
		// full duplex?

		// send to rxclk
		if (received) {
//			emu->out_log(LOG_DEBUG, _T("comm send to acia data=%d pos:%d") ,recv_buff[recv_buff_r_pos],recv_buff_r_pos);

			d_ctrl->write_signal(SIG_ACIA_RXCLK, 1, 1);
			// send data to acia
			d_ctrl->write_signal(SIG_ACIA_RXDATA, recv_buff[recv_buff_r_pos], 1);
			recv_buff_r_pos++;
			if (recv_buff_r_pos >= recv_buff_w_pos) {
				received = false;
			}
		}

		// receive from acia
//		emu->out_debug("comm recv from acia");

		d_ctrl->write_signal(SIG_ACIA_TXCLK, 1, 1);
	}
}

// ----------------------------------------------------------------------------

void COMM::enable_server()
{
	if (!config.comm_server[cfg_num]) {
		// connection active as client
		if (client_ch != -1) {
			// error 
			emu->out_log(LOG_ERROR, _T("Already connecting to another comm server."));
			return;
		}
		// get socket channel
		server_ch = emu->get_socket_channel();
		if (server_ch < 0) {
			return;
		}
		// start server
		if (!emu->init_socket_tcp(server_ch, true)) {
			server_ch = -1;
			emu->out_log(LOG_ERROR, _T("Network socket initialize failed."));
			return;
		}
		if (!emu->connect_socket(server_ch, config.comm_server_host[cfg_num], config.comm_server_port[cfg_num], true)) {
			server_ch = -1;
			emu->out_log(LOG_ERROR, _T("Cannot start as comm server."));
			return;
		}
//		emu->out_log(LOG_DEBUG, _T("Started comm server."));
		config.comm_server[cfg_num] = true;
	} else {
		// stop server
		if (client_ch != -1) {
			// disconnect client
			emu->disconnect_socket(client_ch);
		}
		emu->disconnect_socket(server_ch);
//		emu->out_log(LOG_DEBUG, _T("Stopped comm server."));
		config.comm_server[cfg_num] = false;
	}
}

void COMM::enable_connect()
{
	if (!config.comm_connect[cfg_num]) {
		connect_socket();
	} else {
		disconnect_socket();
	}
}

uint8* COMM::get_sendbuffer(int ch, int* size)
{
//	emu->out_log(LOG_DEBUG, _T("get_sendbuffer: ch:%d client_ch:%d"),ch,client_ch);
	if (ch == client_ch) {
		*size = (send_buff_w_pos - send_buff_r_pos);
//		emu->out_log(LOG_DEBUG, _T("Send buffer. w:%d r:%d size:%d"),send_buff_w_pos,send_buff_r_pos,*size);
		return &send_buff[send_buff_r_pos];
	} else {
		*size = 0;
		return NULL;
	}
}
void COMM::inc_sendbuffer_ptr(int ch, int size)
{
//	emu->out_log(LOG_DEBUG, _T("inc_sendbuffer_ptr: ch:%d client_ch:%d"),ch,client_ch);
	if (ch == client_ch) {
		send_buff_r_pos += size;
//		emu->out_log(LOG_DEBUG, _T("Sent buffer. w:%d r:%d size:%d"),send_buff_w_pos,send_buff_r_pos,size);
		if (send_buff_w_pos <= send_buff_r_pos) {
			// all data sent.
			send_buff_w_pos = 0;
			send_buff_r_pos = 0;
		}
		return;
	}
}
uint8* COMM::get_recvbuffer0(int ch, int* size0, int* size1)
{
//	emu->out_log(LOG_DEBUG, _T("get_recvbuffer0: ch:%d client_ch:%d"),ch,client_ch);
	if (ch == client_ch) {
		*size0 = (COMM_MAX_BUFF - recv_buff_w_pos);
		*size1 = 0;
//		emu->out_log(LOG_DEBUG, _T("Recve buffer. w:%d size:%d"),recv_buff_w_pos,*size0);
		return &recv_buff[recv_buff_w_pos];
	} else {
		*size0 = 0;
		*size1 = 0;
		return NULL;
	}
}

uint8* COMM::get_recvbuffer1(int ch)
{
	if (ch == client_ch) {
		return recv_buff;
	} else {
		return NULL;
	}
}

void COMM::inc_recvbuffer_ptr(int ch, int size)
{
//	emu->out_log(LOG_DEBUG, _T("inc_recvbuffer_ptr: ch:%d client_ch:%d"),ch,client_ch);
	if (ch == client_ch) {
		recv_buff_w_pos += size;
//		emu->out_log(LOG_DEBUG, _T("Recvd buffer. w:%d size:%d"),recv_buff_w_pos,size);
		if (size > 0) {
			received = true;
		}
	}
}

bool COMM::connect_socket()
{
	if (!config.comm_server[cfg_num] && client_ch == -1) {
		// get socket channel
		client_ch = emu->get_socket_channel();
		if (client_ch < 0) {
			return false;
		}
		// connect
		if (!emu->init_socket_tcp(client_ch)) {
			client_ch = -1;
			emu->out_log(LOG_ERROR, _T("Network socket initialize failed."));
			return false;
		}
		if (!emu->connect_socket(client_ch, config.comm_server_host[cfg_num], config.comm_server_port[cfg_num])) {
			client_ch = -1;
			emu->out_log(LOG_ERROR, _T("Cannot connect to comm server."));
			return false;
		}
		connect = 1;
//		emu->out_log(LOG_DEBUG, _T("Connect comm client."));
	}
	return true;
}
void COMM::disconnect_socket()
{
	if (client_ch != -1) {
		// disconnect
		emu->disconnect_socket(client_ch);
//		emu->out_log(LOG_DEBUG, _T("Disconnect comm client."));
	}
}
void COMM::network_connected(int ch)
{
	if (ch == client_ch) {
		connect = 2;
		config.comm_connect[cfg_num] = true;
//		emu->out_log(LOG_DEBUG, _T("Connected comm. ch:%d"),ch);
	}
}
void COMM::network_disconnected(int ch)
{
	if (ch == client_ch) {
		connect = 0;
		config.comm_connect[cfg_num] = false;
//		emu->out_log(LOG_DEBUG, _T("Disonnected comm. ch:%d"),ch);
		client_ch = -1;
	} else if (ch == server_ch) {
//		emu->out_log(LOG_DEBUG, _T("Disonnected comm. ch:%d"),ch);
		server_ch = -1;
	}
}
void COMM::network_writeable(int ch)
{
	if (ch == client_ch) {
//		emu->out_log(LOG_DEBUG, _T("Writeable comm. ch:%d"),ch);
		connect = 3;
	}
}
void COMM::network_readable(int ch)
{
}
void COMM::network_accepted(int ch, int new_ch)
{
	if (ch == server_ch) {
		if (client_ch != -1) {
			// Another client is already connected on my server.
			emu->disconnect_socket(new_ch);
		}
		client_ch = new_ch;
		config.comm_connect[cfg_num] = true;
	}
}

// ----------------------------------------------------------------------------

void COMM::save_state(FILEIO *fp)
{
	struct vm_state_st vm_state;

	//
	vm_state_ident.version = 1;
	vm_state_ident.size = sizeof(vm_state_ident) + sizeof(vm_state);

	// copy values
	memset(&vm_state, 0, sizeof(vm_state));
	vm_state.register_id = register_id;
	vm_state.is_rs = is_rs;

	fp->Fwrite(&vm_state_ident, sizeof(vm_state_ident), 1);
	fp->Fwrite(&vm_state, sizeof(vm_state), 1);
}

void COMM::load_state(FILEIO *fp)
{
	vm_state_ident_t vm_state_i;
	struct vm_state_st vm_state;

	READ_STATE_CHUNK(fp, vm_state_i, vm_state);

	// copy values
	register_id = vm_state.register_id;
	is_rs = vm_state.is_rs;
}
