/** @file keyboard.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.08 -

	@brief [ keyboard ]
*/

#include "../../SimpleIni.h"
#include "keyboard.h"
#include "display.h"
#include "../../gui/ag_keybind_dlg.h"
#include "keyboard_bind.h"

#define KEYBOARD_COUNTER_CYCLE	1
#define KEYBOARD_COUNTER_MAX	(0x80 * KEYBOARD_COUNTER_CYCLE * 2)

#define KEYBIND_HEADER	"KEYBIND3"

void KEYBOARD::initialize()
{
	key_stat = emu->key_buffer();
	kb_mode = 0x44;
	scan_code = 0;
	key_scan_code = 0;
	key_pressed = 0;
	kb_nmi = 0x7f;

	mouse_stat = emu->mouse_buffer();
	lpen_flg = 0;
	lpen_flg_prev = 0;
	lpen_bl = false;

	joy_stat = emu->joy_buffer();

	key_mod = emu->get_key_mod_ptr();
	pause_pressed = false;
	altkey_pressed = false;

	counter = 0;
	remain_count = -1;
//	frame_counter = 0;
//	event_counter = 0;

	memcpy(scan2key_map, scan2key_defmap, sizeof(scan2key_defmap));
	memcpy(scan2joy_map, scan2joy_defmap, sizeof(scan2joy_defmap));
	for(int i=0; i<KEYBIND_PRESETS; i++) {
		memcpy(scan2key_preset_map[i], scan2key_defmap, sizeof(scan2key_defmap));
		memcpy(scan2joy_preset_map[i], scan2joy_defmap, sizeof(scan2joy_defmap));
	}

	// load keybind
	if (load_ini_file() != true) {
		load_cfg_file();
	}

	// for dialog box
	emu->keybindbox->SetVmKeyMap(0, (uint16 *)kb2scan_map, 128);
	emu->keybindbox->SetVmKeyMap(1, (uint16 *)kb2scan_map, 128);
//	emu->keybindbox->SetVmKey(1, 1, 0xff);	// disable break by joypad

	emu->keybindbox->SetVkKeyDefMap(0, (uint32 *)scan2key_defmap, KEYBIND_KEYS, KEYBIND_ASSIGN);
	emu->keybindbox->SetVkKeyDefMap(1, (uint32 *)scan2joy_defmap, KEYBIND_KEYS, KEYBIND_ASSIGN);

	emu->keybindbox->SetVkKeyMap(0, (uint32 *)scan2key_map);
	emu->keybindbox->SetVkKeyMap(1, (uint32 *)scan2joy_map);

	for(int i=0; i<KEYBIND_PRESETS; i++) {
		emu->keybindbox->SetVkKeyPresetMap(0, (uint32 *)scan2key_preset_map[i], i);
		emu->keybindbox->SetVkKeyPresetMap(1, (uint32 *)scan2joy_preset_map[i], i);
	}

	emu->keybindbox->SetTitleLabel(_T("level3 key"),_T("bind"),_T("joypad"));

	// event
	register_frame_event(this);
}

void KEYBOARD::release()
{
	// save keybind
	save_keybind();
}

// ----------------------------------------------------------------------------
void KEYBOARD::save_keybind()
{
	// save keybind
	save_cfg_file();
	save_ini_file();
}

bool KEYBOARD::load_cfg_file()
{
	_TCHAR *app_path, file_path[_MAX_PATH];
	char header[10];

	uint16 scan_map_old16[KEYBIND_KEYS][KEYBIND_ASSIGN];

	FILEIO* fio = new FILEIO();
	app_path = emu->application_path();

	_stprintf(file_path, _T("%skeybind.cfg"), app_path);
	if(fio->Fopen(file_path, FILEIO_READ_BINARY)) {
		fio->Fread(header, sizeof(char), 8);
		if (strncmp(header, KEYBIND_HEADER, 7) == 0) {
#if 0
			fio->Fread(scan2key_map, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			for(int i=0; i<KEYBIND_PRESETS; i++) {
				fio->Fread(scan2key_preset_map[i], sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			}
			fio->Fread(scan2joy_map, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			for(int i=0; i<KEYBIND_PRESETS; i++) {
				fio->Fread(scan2joy_preset_map[i], sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			}
#else
			fio->Fread(scan_map_old16, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			for(int kk=0; kk<KEYBIND_KEYS; kk++) {
				for(int ka=0; ka<KEYBIND_ASSIGN; ka++) {
					scan2key_map[kk][ka] = scan_map_old16[kk][ka];
				}
			}
			for(int i=0; i<KEYBIND_PRESETS; i++) {
				fio->Fread(scan_map_old16, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
				for(int kk=0; kk<KEYBIND_KEYS; kk++) {
					for(int ka=0; ka<KEYBIND_ASSIGN; ka++) {
						scan2key_preset_map[i][kk][ka] = scan_map_old16[kk][ka];
					}
				}
			}
			fio->Fread(scan_map_old16, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
			for(int kk=0; kk<KEYBIND_KEYS; kk++) {
				for(int ka=0; ka<KEYBIND_ASSIGN; ka++) {
					scan2joy_map[kk][ka] = scan_map_old16[kk][ka];
				}
			}
			for(int i=0; i<KEYBIND_PRESETS; i++) {
				fio->Fread(scan_map_old16, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
				for(int kk=0; kk<KEYBIND_KEYS; kk++) {
					for(int ka=0; ka<KEYBIND_ASSIGN; ka++) {
						scan2joy_preset_map[i][kk][ka] = scan_map_old16[kk][ka];
					}
				}
			}
#endif
			emu->out_log(LOG_INFO,_T("keybind.cfg was loaded."));
		} else {
			emu->out_log(LOG_WARN,_T("keybind.cfg was invalid version."));
		}
		fio->Fclose();
//	} else {
//		emu->out_log(LOG_WARN,_T("keybind.cfg couldn't be loaded."));
	}
	delete fio;

	// convert keybind map from old version to current version
	if (header[7] == '1' || header[7] == '2') {
		emu->out_log(LOG_WARN,_T("keybind.cfg is old version."));
		for (int n=0x34; n<=0x36; n+=2) {
			for (int j=0; j<2; j++) {
				scan2key_map[n][j] = scan2key_defmap[n][j];
				scan2joy_map[n][j] = scan2joy_defmap[n][j];
				for (int i=0; i<KEYBIND_PRESETS; i++) {
					scan2key_preset_map[i][n][j] = scan2key_defmap[n][j];
					scan2joy_preset_map[i][n][j] = scan2joy_defmap[n][j];
				}
			}
		}
	}
	if (header[7] == '1') {
		for (int n=0x60; n<=0x80; n++) {
			for (int j=0; j<2; j++) {
				scan2key_map[n][j] = scan2key_defmap[n][j];
				scan2joy_map[n][j] = scan2joy_defmap[n][j];
				for (int i=0; i<KEYBIND_PRESETS; i++) {
					scan2key_preset_map[i][n][j] = scan2key_defmap[n][j];
					scan2joy_preset_map[i][n][j] = scan2joy_defmap[n][j];
				}
			}
		}
	}
	return true;
}

void KEYBOARD::save_cfg_file()
{
#if 0
	_TCHAR app_path[_MAX_PATH], file_path[_MAX_PATH];
	char header[10];

	emu->application_path(app_path);
	FILEIO* fio = new FILEIO();

	strcpy(header, KEYBIND_HEADER);
	_stprintf(file_path, _T("%skeybind.cfg"), app_path);
	if(fio->Fopen(file_path, FILEIO_WRITE_BINARY)) {
		fio->Fwrite(header, sizeof(char), 8);
		fio->Fwrite(scan2key_map, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
		for(int i=0; i<4; i++) {
			fio->Fwrite(scan2key_preset_map[i], sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
		}
		fio->Fwrite(scan2joy_map, sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
		for(int i=0; i<4; i++) {
			fio->Fwrite(scan2joy_preset_map[i], sizeof(uint16), KEYBIND_KEYS * KEYBIND_ASSIGN);
		}
		fio->Fclose();
		emu->out_log(LOG_INFO,_T("keybind.cfg was saved."));
	} else {
		emu->out_log(LOG_WARN,_T("keybind.cfg couldn't be saved."));
	}
	delete fio;
#endif
}

// ----------------------------------------------------------------------------
bool KEYBOARD::load_ini_file()
{
	_TCHAR *app_path, file_path[_MAX_PATH];
	_TCHAR section[100];

	CSimpleIni *ini = new CSimpleIni();

	// load ini file
	app_path = emu->application_path();

	_stprintf(file_path, _T("%skeybind.ini"), app_path);
	if (ini->LoadFile(file_path) == SI_OK) {
		emu->out_log(LOG_INFO,_T("keybind.ini was loaded."));
	} else {
		emu->out_log(LOG_WARN,_T("keybind.ini couldn't be loaded."));
		delete ini;
		return false;
	}

	CSimpleIni::TNamesDepend keys;
	CSimpleIni::TNamesDepend::const_iterator ik;

	_stprintf(section, _T("Keyboard"));
	ini->GetAllKeys(section, keys);
	if (!keys.empty()) {
		memset(scan2key_map, 0, sizeof(scan2key_map));
	}
	for (ik = keys.begin(); ik != keys.end(); ik++) {
		int k = 0;
		int i = 0;
		int rc = _stscanf(ik->pItem, _T("%02x_%d"), &k, &i);
		if (rc != 2 || k < 0 || k >= KEYBIND_KEYS || i < 0 || i >= KEYBIND_ASSIGN) {
			continue;
		}
		long v = ini->GetLongValue(section, ik->pItem);
		scan2key_map[k][i] = (uint32)(v & 0xffffffff);
	}
	for (int no = 0; no < 4; no++) {
		_stprintf(section, _T("KeyboardPreset%d"), no + 1);
		ini->GetAllKeys(section, keys);
		if (!keys.empty()) {
			memset(scan2key_preset_map[no], 0, sizeof(scan2key_preset_map[no]));
		}
		for (ik = keys.begin(); ik != keys.end(); ik++) {
			int k = 0;
			int i = 0;
			int rc = _stscanf(ik->pItem, _T("%02x_%d"), &k, &i);
			if (rc != 2 || k < 0 || k >= KEYBIND_KEYS || i < 0 || i >= KEYBIND_ASSIGN) {
				continue;
			}
			long v = ini->GetLongValue(section, ik->pItem);
			scan2key_preset_map[no][k][i] = (uint32)(v & 0xffffffff);
		}
	}
	_stprintf(section, _T("Joypad"));
	ini->GetAllKeys(section, keys);
	if (!keys.empty()) {
		memset(scan2joy_map, 0, sizeof(scan2joy_map));
	}
    for (ik = keys.begin(); ik != keys.end(); ik++) {
		int k = 0;
		int i = 0;
		int rc = _stscanf(ik->pItem, _T("%02x_%d"), &k, &i);
		if (rc != 2 || k < 0 || k >= KEYBIND_KEYS || i < 0 || i >= KEYBIND_ASSIGN) {
			continue;
		}
		long v = ini->GetLongValue(section, ik->pItem);
		scan2joy_map[k][i] = (uint32)(v & 0xffffffff);
	}
	for (int no = 0; no < 4; no++) {
		_stprintf(section, _T("JoypadPreset%d"), no + 1);
		ini->GetAllKeys(section, keys);
		if (!keys.empty()) {
			memset(scan2joy_preset_map[no], 0, sizeof(scan2joy_preset_map[no]));
		}
		for (ik = keys.begin(); ik != keys.end(); ik++) {
			int k = 0;
			int i = 0;
			int rc = _stscanf(ik->pItem, _T("%02x_%d"), &k, &i);
			if (rc != 2 || k < 0 || k >= KEYBIND_KEYS || i < 0 || i >= KEYBIND_ASSIGN) {
				continue;
			}
			long v = ini->GetLongValue(section, ik->pItem);
			scan2joy_preset_map[no][k][i] = (uint32)(v & 0xffffffff);
		}
	}

	keys.clear();

	delete ini;

	return true;
}

void KEYBOARD::save_ini_file()
{
	_TCHAR *app_path, file_path[_MAX_PATH];
	_TCHAR comment[100];
	_TCHAR section[100];
	_TCHAR key[100];

	CSimpleIni *ini = new CSimpleIni();

	// section
	_stprintf(comment, _T("; %s keybind file"), _T(DEVICE_NAME));
	ini->SetLongValue(_T(""), _T("Version"), 1, comment, false);

	_stprintf(section, _T("Keyboard"));
    for (int k = 0; k < KEYBIND_KEYS; k++) {
		for (int i = 0; i < KEYBIND_ASSIGN; i++) {
			_stprintf(key, _T("%02x_%d"), k, i);
			if (scan2key_map[k][i] != 0) {
				ini->SetLongValue(section, key, scan2key_map[k][i], NULL, true);
			}
		}
	}
    for (int no = 0; no < 4; no++) {
		_stprintf(section, _T("KeyboardPreset%d"), no + 1);
		for (int k = 0; k < KEYBIND_KEYS; k++) {
			for (int i = 0; i < KEYBIND_ASSIGN; i++) {
				_stprintf(key, _T("%02x_%d"), k, i);
				if (scan2key_preset_map[no][k][i] != 0) {
					ini->SetLongValue(section, key, scan2key_preset_map[no][k][i], NULL, true);
				}
			}
		}
	}
	_stprintf(section, _T("Joypad"));
    for (int k = 0; k < KEYBIND_KEYS; k++) {
		for (int i = 0; i < KEYBIND_ASSIGN; i++) {
			_stprintf(key, _T("%02x_%d"), k, i);
			if (scan2joy_map[k][i] != 0) {
				ini->SetLongValue(section, key, scan2joy_map[k][i], NULL, true);
			}
		}
	}
    for (int no = 0; no < 4; no++) {
		_stprintf(section, _T("JoypadPreset%d"), no + 1);
		for (int k = 0; k < KEYBIND_KEYS; k++) {
			for (int i = 0; i < KEYBIND_ASSIGN; i++) {
				_stprintf(key, _T("%02x_%d"), k, i);
				if (scan2joy_preset_map[no][k][i] != 0) {
					ini->SetLongValue(section, key, scan2joy_preset_map[no][k][i], NULL, true);
				}
			}
		}
	}

	// save ini file
	app_path = emu->application_path();

	_stprintf(file_path, _T("%skeybind.ini"), app_path);
	if (ini->SaveFile(file_path) == SI_OK) {
		emu->out_log(LOG_INFO,_T("keybind.ini was saved."));
	} else {
		emu->out_log(LOG_WARN,_T("keybind.ini couldn't be saved."));
	}

	delete ini;
}

// ----------------------------------------------------------------------------

void KEYBOARD::reset()
{
	kb_mode = 0;
	scan_code = 0;
	key_scan_code = 0;
	key_pressed = 0;
	kb_nmi = 0x7f;

	lpen_flg = 0;
	lpen_flg_prev = 0;
	lpen_bl = false;

	counter = 0;
	remain_count = -1;
//	frame_counter = 0;
//	event_counter = 0;
}

void KEYBOARD::write_signal(int id, uint32 data, uint32 mask)
{
	switch (id) {
		case SIG_KEYBOARD_HSYNC:
			if (data & mask) update_keyboard();
			break;
		case SIG_CPU_RESET:
			now_reset = (data & mask) ? true : false;
			reset();
			break;
	}
}

void KEYBOARD::write_io8(uint32 addr, uint32 data)
{
	if (now_reset) return;

	switch (addr & 0xffff) {
		case ADDR_LPEN_BL:
			lpen_bl = (data & 0x80) ? true : false;
//			emu->out_debug("w_lpenbl: %02x",lpen_bl);
			break;
		case ADDR_KEYBOARD:
			kb_mode = data;

			// keyborad irq interrupt
			if ((kb_mode & 0x40) && (key_scan_code & 0x80) != 0) {
				d_cpu->write_signal(SIG_CPU_IRQ, 1,1);
			}
//			emu->out_debug("w_kb: fr:%04d ct:%03x kb:%02x"
//			, frame_counter, counter, kb_mode);
			break;
	}
}

uint32 KEYBOARD::read_io8(uint32 addr)
{
	uint32 data = 0;

	if (now_reset) return data;

	switch (addr & 0xffff) {
		case ADDR_KB_NMI:
			// break key (0xffc8)
			data = kb_nmi; // & 0x80;
			kb_nmi = 0x7f;
			break;
		case ADDR_LP_FLG:
			// light pen flag (0xffcb)
			data = lpen_flg & 0x80;
			lpen_flg = 0;

//			emu->out_debug("r_lpenflg:%04d ct:%03x d:%02x"
//			, frame_counter, counter, data);
			break;
		case ADDR_KEYBOARD:
			// keyboard (0xffe0)
			data = key_scan_code;
			key_scan_code &= 0x7f;

			emu->recognized_key(key_native_code);
			key_native_code= 0;

//			emu->out_debug("r_ks fr:%04d ct:%03x rc:%02d d:%02x k:%02x"
//			, frame_counter, counter, remain_count, data, key_scan_code);
			break;
	}
	return data;
}

void KEYBOARD::clear_scan_code(int key_counter)
{
//	if (kb_mode & 0x40) {
//		key_counter = (int)(key_counter / KEYBOARD_COUNTER_CYCLE);
//	}

	// if kb bit3 is set, enable scan_code is between 0x00 and 0x07
	scan_code = (key_counter) & ((kb_mode & 0x08) ? 0x07 : 0x7f);
	if (key_scan_code < 0x80) {
		key_scan_code = scan_code;
	}
}

void KEYBOARD::update_scan_code(int key_counter)
{
	bool pressed = false;

	if (scan_code == 0x7f) {
		if (key_pressed == 1) {
			pressed = true;
			scan_code |= 0x80;
			remain_count = KEYBOARD_COUNTER_MAX * 8;
		}
		if (key_pressed > 0) key_pressed--;
	} else if (scan_code >= 0x60) {
		// no assaign
	} else {
		pressed = pressing_key(scan_code);
		if (pressed == true) {
			scan_code |= 0x80;
			key_pressed = 2;
			remain_count = KEYBOARD_COUNTER_MAX * 8;
		}
	}

	if (key_scan_code < 0x80) {
		key_scan_code = scan_code;
	} else if (scan_code >= 0x80) {
		key_scan_code = scan_code;
	}

	// keyborad irq interrupt
	if ((kb_mode & 0x40) && (pressed == true)) {
		d_cpu->write_signal(SIG_CPU_IRQ, 1,1);
	}

//	pressed = false;

//	emu->out_debug("kbd fr:%04d ct:%03x rc:%02d s:%02x k:%02x"
//	, frame_counter, counter, remain_count, scan_code, key_scan_code);
}

void KEYBOARD::update_special_key()
{
	bool pressed = false;

	for(int code = 0x7f; code <= 0x80; code++) {
		pressed = pressing_key(code);
		switch(code) {
			case 0x7f:
				// reset switch
				if (pressed != now_reset) {
					d_board->write_signal(SIG_CPU_RESET, pressed ? 1 : 0, 1);
				}
				break;
			case 0x80:
				// break key
				if (pressed == true) {
					if (!(kb_nmi & 0x80)) {
						kb_nmi = 0xff;
						// keyborad nmi interrupt
						if ((kb_mode & 0x80)) {
							d_cpu->write_signal(SIG_CPU_NMI, 1,1);
						}
					}
				} else {
					kb_nmi = 0x7f;
				}
				break;
		}
	}
}

void KEYBOARD::update_light_pen()
{
	if (lpen_bl) {
		lpen_flg = (mouse_stat[2] & 1) ? 0x80 : 0;
		if (lpen_flg != 0) {
			// push light pen
			// LPSTB signal
			d_disp->write_signal(SIG_DISPLAY_LPSTB
				,((mouse_stat[0] & 0xffff) << 16) | (mouse_stat[1] & 0xffff)
				, 0xffffffff);

			// IRQ interrupt
			d_cpu->write_signal(SIG_CPU_IRQ, 1,1);

		}
		lpen_flg_prev = lpen_flg;

	}
//	emu->out_debug("lpen bl:%02x flg:%02x x:%03d y:%03d reg:%02x%02x",lpen_bl,lpen_flg, mouse_stat[0], mouse_stat[1], regs[16], regs[17]);
}

void KEYBOARD::update_system_key()
{
	bool pressed = false;

	for(int code = 0x60; code <= 0x61; code++) {
		pressed = pressing_key(code);
		switch(code) {
			case 0x60:
				// pause
				if (pressed && pause_pressed == false) {
					emu->set_pause(2, emu->get_pause(2) ? false : true);
				}
				pause_pressed = pressed;
				break;
			case 0x61:
				// alt
				altkey_pressed = pressed;
				*key_mod = (altkey_pressed) ? (*key_mod) | KEY_MOD_ALT_KEY : (*key_mod) & ~KEY_MOD_ALT_KEY;
				break;
		}
	}
}

// ----------------------------------------------------------------------------

inline bool KEYBOARD::pressing_key(int key_code)
{
	uint32 code = 0;
	bool pressed = false;
	int i = 0;

	for(i=0; i<2; i++) {
		code = scan2key_map[key_code][i];
		if (code == 0) break;

		// key pressed ?
		if (key_stat[code]) {
			pressed = true;
			break;
		}
	}
	if (pressed != true) {
		for(i=0; i<2; i++) {
			code = scan2joy_map[key_code][i];
			if (code == 0) continue;

			// joypad pressed ?
			// allow key or button
			if (
				((scan2joy_map[0x81][0] & 1) == 0 && ((joy_stat[i] & 0xf & code) == code || (joy_stat[i] & 0xfffffff0 & code) == code))
			||  ((scan2joy_map[0x81][0] & 1) != 0 && ((joy_stat[i] & 0xf) == code || (joy_stat[i] & 0xfffffff0) == code))
			) {
				pressed = true;
				break;
			}
		}
	}
	if (pressed) {
		key_native_code = code & 0xffff;
	}
	return pressed;
}

// ----------------------------------------------------------------------------
// event handler
// ----------------------------------------------------------------------------

void KEYBOARD::event_frame()
{
	// reset switch & break key
	update_special_key();
	// other system keys
	update_system_key();
	// Cgy
	update_light_pen();
}

void KEYBOARD::event_callback(int event_id, int err)
{
}

// ----------------------------------------------------------------------------
void KEYBOARD::update_keyboard()
{
//	if (event_counter == 0) {
//		frame_counter = (frame_counter + 1) % 10000;
//	}
//	event_counter++;
//	if (event_counter >= KEYBOARD_COUNTER_MAX) event_counter = 0;

	if (remain_count == 0) {
		key_scan_code &= 0x7f;
		counter = (counter - 2) % KEYBOARD_COUNTER_MAX;
	}
	if (remain_count >= 0) remain_count--;

	// L[ĂȂƂɃL[XL
	if ((key_scan_code & 0x80) == 0) {
		if ((counter & 1) == 0) {
			clear_scan_code(counter >> 1);
		} else {
			update_scan_code(counter >> 1);
		}
		if (!now_reset) counter++;
		if (counter >= KEYBOARD_COUNTER_MAX) counter = 0;
	}
}

// ----------------------------------------------------------------------------

void KEYBOARD::save_state(FILEIO *fio)
{
	struct vm_state_st vm_state;

	//
	vm_state_ident.version = 1;
	vm_state_ident.size = sizeof(vm_state_ident) + sizeof(vm_state);

	// copy values
	memset(&vm_state, 0, sizeof(vm_state));
	vm_state.kb_mode = kb_mode;
	vm_state.kb_nmi = kb_nmi;
	vm_state.lpen_flg = lpen_flg;
	vm_state.lpen_flg_prev = lpen_flg_prev;
	vm_state.lpen_bl = lpen_bl ? 1 : 0;

	fio->Fwrite(&vm_state_ident, sizeof(vm_state_ident), 1);
	fio->Fwrite(&vm_state, sizeof(vm_state), 1);
}

void KEYBOARD::load_state(FILEIO *fio)
{
	vm_state_ident_t vm_state_i;
	struct vm_state_st vm_state;

	READ_STATE_CHUNK(fio, vm_state_i, vm_state);

	kb_mode = vm_state.kb_mode;
	kb_nmi = vm_state.kb_nmi;
	lpen_flg = vm_state.lpen_flg;
	lpen_flg_prev = vm_state.lpen_flg_prev;
	lpen_bl = vm_state.lpen_bl ? true : false;
}
